@@ -11,7 +11,6 @@ if [ "${TARGET}" = "linux" ]; then
         python3-rdflib \
         libgtk2.0-dev \
         libgtk-3-dev \
-        libqt4-dev \
         qtbase5-dev \
         libasound2-dev \
         libpulse-dev \
@@ -42,7 +41,6 @@ elif [ "${TARGET}" = "linux-strict" ]; then
         python3-rdflib \
         libgtk2.0-dev \
         libgtk-3-dev \
-        libqt4-dev \
         qtbase5-dev \
         libasound2-dev \
         libpulse-dev \
@@ -66,7 +64,6 @@ elif [ "${TARGET}" = "linux-juce-strict" ]; then
         python3-rdflib \
         libgtk2.0-dev \
         libgtk-3-dev \
-        libqt4-dev \
         qtbase5-dev \
         libasound2-dev \
         libjack-jackd2-dev \
@@ -91,17 +88,29 @@ elif [ "${TARGET}" = "macos" ]; then
         pkg-config \
         apple-x86-setup
 
+elif [ "${TARGET}" = "macos-native" ] || [ "${TARGET}" = "macos-universal" ]; then
+    HOMEBREW_NO_AUTO_UPDATE=1 brew install cmake jq meson
+    exit 0
+fi
+
 elif [ "${TARGET}" = "win32" ]; then
     sudo apt-get install -y \
-        mingw32-x-gcc \
-        mingw32-x-pkgconfig
+        g++-multilib \
+        mingw-w64 \
+        binutils-mingw-w64-i686 \
+        binutils-mingw-w64-x86-64 \
+        g++-mingw-w64-i686 \
+        g++-mingw-w64-x86-64 \
+        wine-devel-dev \
+        winehq-stable
 
 elif [ "${TARGET}" = "win64" ]; then
     sudo apt-get install -y \
-        mingw32-x-gcc \
-        mingw32-x-pkgconfig \
-        mingw64-x-gcc \
-        mingw64-x-pkgconfig
+        mingw-w64 \
+        binutils-mingw-w64-x86-64 \
+        g++-mingw-w64-x86-64 \
+        wine-devel-dev \
+        winehq-stable
 
 elif [ "${TARGET}" = "pylint" ]; then
     sudo apt-get install -y \
