@@ -16,11 +16,14 @@
  */
 
 #include "CarlaEnginePorts.hpp"
-#include "CarlaEngineGraph.hpp"
 #include "CarlaEngineUtils.hpp"
 #include "CarlaMathUtils.hpp"
 #include "CarlaMIDI.h"
 
+#ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
+#include "CarlaEngineGraph.hpp"
+#endif
+
 #include "lv2/lv2.h"
 
 CARLA_BACKEND_START_NAMESPACE
@@ -28,8 +31,9 @@ CARLA_BACKEND_START_NAMESPACE
 // -----------------------------------------------------------------------
 // Fallback data
 
-static const EngineEvent kFallbackEngineEvent = { kEngineEventTypeNull, 0, 0, {{ kEngineControlEventTypeNull, 0, 0.0f }} };
-//static CarlaEngineEventCV kFallbackEngineEventCV = { nullptr, (uint32_t)-1, 0.0f };
+static EngineEvent kFallbackEngineEvent = {
+    kEngineEventTypeNull, 0, 0, {{ kEngineControlEventTypeNull, 0, -1, 0.0f, true }}
+};
 
 // -----------------------------------------------------------------------
 // Carla Engine port (Abstract)
@@ -166,7 +170,7 @@ uint32_t CarlaEngineEventPort::getEventCount() const noexcept
     return i;
 }
 
-const EngineEvent& CarlaEngineEventPort::getEvent(const uint32_t index) const noexcept
+EngineEvent& CarlaEngineEventPort::getEvent(const uint32_t index) const noexcept
 {
     CARLA_SAFE_ASSERT_RETURN(kIsInput, kFallbackEngineEvent);
     CARLA_SAFE_ASSERT_RETURN(fBuffer != nullptr, kFallbackEngineEvent);
@@ -176,24 +180,28 @@ const EngineEvent& CarlaEngineEventPort::getEvent(const uint32_t index) const no
     return fBuffer[index];
 }
 
-const EngineEvent& CarlaEngineEventPort::getEventUnchecked(const uint32_t index) const noexcept
+EngineEvent& CarlaEngineEventPort::getEventUnchecked(const uint32_t index) const noexcept
 {
     return fBuffer[index];
 }
 
 bool CarlaEngineEventPort::writeControlEvent(const uint32_t time, const uint8_t channel, const EngineControlEvent& ctrl) noexcept
 {
-    return writeControlEvent(time, channel, ctrl.type, ctrl.param, ctrl.value);
+    return writeControlEvent(time, channel, ctrl.type, ctrl.param, ctrl.midiValue, ctrl.normalizedValue);
 }
 
-bool CarlaEngineEventPort::writeControlEvent(const uint32_t time, const uint8_t channel, const EngineControlEventType type, const uint16_t param, const float value) noexcept
+bool CarlaEngineEventPort::writeControlEvent(const uint32_t time, const uint8_t channel,
+                                             const EngineControlEventType type,
+                                             const uint16_t param,
+                                             const int8_t midiValue,
+                                             const float normalizedValue) noexcept
 {
     CARLA_SAFE_ASSERT_RETURN(! kIsInput, false);
     CARLA_SAFE_ASSERT_RETURN(fBuffer != nullptr, false);
     CARLA_SAFE_ASSERT_RETURN(kProcessMode != ENGINE_PROCESS_MODE_SINGLE_CLIENT && kProcessMode != ENGINE_PROCESS_MODE_MULTIPLE_CLIENTS, false);
     CARLA_SAFE_ASSERT_RETURN(type != kEngineControlEventTypeNull, false);
     CARLA_SAFE_ASSERT_RETURN(channel < MAX_MIDI_CHANNELS, false);
-    CARLA_SAFE_ASSERT(value >= 0.0f && value <= 1.0f);
+    CARLA_SAFE_ASSERT(normalizedValue >= 0.0f && normalizedValue <= 1.0f);
 
     if (type == kEngineControlEventTypeParameter) {
         CARLA_SAFE_ASSERT(! MIDI_IS_CONTROL_BANK_SELECT(param));
@@ -210,9 +218,10 @@ bool CarlaEngineEventPort::writeControlEvent(const uint32_t time, const uint8_t
         event.time    = time;
         event.channel = channel;
 
-        event.ctrl.type  = type;
-        event.ctrl.param = param;
-        event.ctrl.value = carla_fixedValue<float>(0.0f, 1.0f, value);
+        event.ctrl.type            = type;
+        event.ctrl.param           = param;
+        event.ctrl.midiValue       = midiValue;
+        event.ctrl.normalizedValue = carla_fixedValue<float>(0.0f, 1.0f, normalizedValue);
 
         return true;
     }
@@ -262,24 +271,30 @@ bool CarlaEngineEventPort::writeMidiEvent(const uint32_t time, const uint8_t cha
             case MIDI_CONTROL_BANK_SELECT:
             case MIDI_CONTROL_BANK_SELECT__LSB:
                 CARLA_SAFE_ASSERT_RETURN(size >= 3, true);
-                event.type       = kEngineEventTypeControl;
-                event.ctrl.type  = kEngineControlEventTypeMidiBank;
-                event.ctrl.param = data[2];
-                event.ctrl.value = 0.0f;
+                event.type                 = kEngineEventTypeControl;
+                event.ctrl.type            = kEngineControlEventTypeMidiBank;
+                event.ctrl.param           = data[2];
+                event.ctrl.midiValue       = -1;
+                event.ctrl.normalizedValue = 0.0f;
+                event.ctrl.handled         = true;
                 return true;
 
             case MIDI_CONTROL_ALL_SOUND_OFF:
-                event.type       = kEngineEventTypeControl;
-                event.ctrl.type  = kEngineControlEventTypeAllSoundOff;
-                event.ctrl.param = 0;
-                event.ctrl.value = 0.0f;
+                event.type                 = kEngineEventTypeControl;
+                event.ctrl.type            = kEngineControlEventTypeAllSoundOff;
+                event.ctrl.param           = 0;
+                event.ctrl.midiValue       = -1;
+                event.ctrl.normalizedValue = 0.0f;
+                event.ctrl.handled         = true;
                 return true;
 
             case MIDI_CONTROL_ALL_NOTES_OFF:
-                event.type       = kEngineEventTypeControl;
-                event.ctrl.type  = kEngineControlEventTypeAllNotesOff;
-                event.ctrl.param = 0;
-                event.ctrl.value = 0.0f;
+                event.type                 = kEngineEventTypeControl;
+                event.ctrl.type            = kEngineControlEventTypeAllNotesOff;
+                event.ctrl.param           = 0;
+                event.ctrl.midiValue       = -1;
+                event.ctrl.normalizedValue = 0.0f;
+                event.ctrl.handled         = true;
                 return true;
             }
         }
@@ -288,10 +303,12 @@ bool CarlaEngineEventPort::writeMidiEvent(const uint32_t time, const uint8_t cha
         {
             CARLA_SAFE_ASSERT_RETURN(size >= 2, true);
 
-            event.type       = kEngineEventTypeControl;
-            event.ctrl.type  = kEngineControlEventTypeMidiProgram;
-            event.ctrl.param = data[1];
-            event.ctrl.value = 0.0f;
+            event.type                 = kEngineEventTypeControl;
+            event.ctrl.type            = kEngineControlEventTypeMidiProgram;
+            event.ctrl.param           = data[1];
+            event.ctrl.midiValue       = -1;
+            event.ctrl.normalizedValue = 0.0f;
+            event.ctrl.handled         = true;
             return true;
         }
 
@@ -339,11 +356,13 @@ CarlaEngineCVSourcePorts::~CarlaEngineCVSourcePorts()
     delete pData;
 }
 
-bool CarlaEngineCVSourcePorts::addCVSource(CarlaEngineCVPort* const port, const uint32_t portIndexOffset)
+bool CarlaEngineCVSourcePorts::addCVSource(CarlaEngineCVPort* const port,
+                                           const uint32_t portIndexOffset,
+                                           const bool reconfigureNow)
 {
     CARLA_SAFE_ASSERT_RETURN(port != nullptr, false);
     CARLA_SAFE_ASSERT_RETURN(port->isInput(), false);
-    carla_debug("CarlaEngineCVSourcePorts::addCVSource(%p)", port);
+    carla_debug("CarlaEngineCVSourcePorts::addCVSource(%p, %u)", port, portIndexOffset);
 
     {
         const CarlaRecursiveMutexLocker crml(pData->rmutex);
@@ -352,7 +371,7 @@ bool CarlaEngineCVSourcePorts::addCVSource(CarlaEngineCVPort* const port, const
         if (! pData->cvs.add(ecv))
             return false;
 
-        if (pData->graph != nullptr && pData->plugin != nullptr)
+        if (reconfigureNow && pData->graph != nullptr && pData->plugin.get() != nullptr)
             pData->graph->reconfigureForCV(pData->plugin, static_cast<uint>(pData->cvs.size()-1), true);
     }
 
@@ -375,7 +394,7 @@ bool CarlaEngineCVSourcePorts::removeCVSource(const uint32_t portIndexOffset)
                 delete ecv.cvPort;
                 pData->cvs.remove(i);
 
-                if (pData->graph != nullptr && pData->plugin != nullptr)
+                if (pData->graph != nullptr && pData->plugin.get() != nullptr)
                     pData->graph->reconfigureForCV(pData->plugin, static_cast<uint>(i), false);
 
                 carla_stdout("found cv source to remove %u", portIndexOffset);
@@ -449,9 +468,10 @@ void CarlaEngineCVSourcePorts::initPortBuffers(const float* const* const buffers
                 event.time    = eventFrame;
                 event.channel = kEngineEventNonMidiChannel;
 
-                event.ctrl.type  = kEngineControlEventTypeParameter;
-                event.ctrl.param = static_cast<uint16_t>(ecv.indexOffset);
-                event.ctrl.value = carla_fixedValue(0.0f, 1.0f, (v - min) / (max - min));
+                event.ctrl.type            = kEngineControlEventTypeParameter;
+                event.ctrl.param           = static_cast<uint16_t>(ecv.indexOffset);
+                event.ctrl.midiValue       = -1;
+                event.ctrl.normalizedValue = carla_fixedValue(0.0f, 1.0f, (v - min) / (max - min));
             }
 
             ecv.previousValue = previousValue;
