@@ -21,6 +21,7 @@
 
 from PyQt5.QtCore import pyqtSlot, qCritical, qFatal, qWarning, QObject
 from PyQt5.QtCore import QPointF, QRectF, QTimer
+from PyQt5.QtWidgets import QGraphicsObject
 
 # ------------------------------------------------------------------------------------------------------------
 # Imports (Custom)
@@ -85,7 +86,12 @@ class CanvasObject(QObject):
             canvas.animation_list.remove(animation)
             item = animation.item()
             if item:
-                item.hide()
+                if isinstance(item, QGraphicsObject):
+                    item.blockSignals(True)
+                    item.hide()
+                    item.blockSignals(False)
+                else:
+                    item.hide()
 
     @pyqtSlot()
     def AnimationFinishedDestroy(self):
@@ -138,6 +144,36 @@ class CanvasObject(QObject):
 
         CanvasCallback(ACTION_PORTS_DISCONNECT, connectionId, 0, "")
 
+    @pyqtSlot(int, bool, int, int)
+    def boxPositionChanged(self, groupId, split, x, y):
+        x2 = y2 = 0
+
+        if split:
+            for group in canvas.group_list:
+                if group.group_id == groupId:
+                    if group.split:
+                        pos = group.widgets[1].pos()
+                        x2  = pos.x()
+                        y2  = pos.y()
+                    break
+
+        valueStr = "%i:%i:%i:%i" % (x, y, x2, y2)
+        CanvasCallback(ACTION_GROUP_POSITION, groupId, 0, valueStr)
+
+    @pyqtSlot(int, bool, int, int)
+    def sboxPositionChanged(self, groupId, split, x2, y2):
+        x = y = 0
+
+        for group in canvas.group_list:
+            if group.group_id == groupId:
+                pos = group.widgets[0].pos()
+                x = pos.x()
+                y = pos.y()
+                break
+
+        valueStr = "%i:%i:%i:%i" % (x, y, x2, y2)
+        CanvasCallback(ACTION_GROUP_POSITION, groupId, 0, valueStr)
+
 # ------------------------------------------------------------------------------------------------------------
 
 def getStoredCanvasPosition(key, fallback_pos):
@@ -201,11 +237,17 @@ def clear():
     if canvas.debug:
         print("PatchCanvas::clear()")
 
+    group_pos = {}
     group_list_ids = []
     port_list_ids = []
     connection_list_ids = []
 
     for group in canvas.group_list:
+        group_pos[group.group_name] = (
+            group.split,
+            group.widgets[0].pos(),
+            group.widgets[1].pos() if group.split else None,
+        )
         group_list_ids.append(group.group_id)
 
     for port in canvas.port_list:
@@ -230,6 +272,7 @@ def clear():
     canvas.port_list = []
     canvas.connection_list = []
     canvas.group_plugin_map = {}
+    canvas.old_group_pos = group_pos
 
     canvas.scene.clearSelection()
 
@@ -276,7 +319,9 @@ def addGroup(group_id, group_name, split=SPLIT_UNDEF, icon=ICON_APPLICATION):
         if group.group_id == group_id:
             qWarning("PatchCanvas::addGroup(%i, %s, %s, %s) - group already exists" % (
                      group_id, group_name.encode(), split2str(split), icon2str(icon)))
-            return
+            return None
+
+    old_matching_group = canvas.old_group_pos.pop(group_name, None)
 
     if split == SPLIT_UNDEF:
         isHardware = bool(icon == ICON_HARDWARE)
@@ -285,8 +330,12 @@ def addGroup(group_id, group_name, split=SPLIT_UNDEF, icon=ICON_APPLICATION):
             split = getStoredCanvasSplit(group_name, SPLIT_YES if isHardware else split)
         elif isHardware:
             split = SPLIT_YES
+        elif old_matching_group is not None and old_matching_group[0]:
+            split = SPLIT_YES
 
     group_box = CanvasBox(group_id, group_name, icon)
+    group_box.positionChanged.connect(canvas.qobject.boxPositionChanged)
+    group_box.blockSignals(True)
 
     group_dict = group_dict_t()
     group_dict.group_id = group_id
@@ -303,18 +352,24 @@ def addGroup(group_id, group_name, split=SPLIT_UNDEF, icon=ICON_APPLICATION):
 
         if features.handle_group_pos:
             group_box.setPos(getStoredCanvasPosition(group_name + "_OUTPUT", CanvasGetNewGroupPos(False)))
+        elif old_matching_group is not None:
+            group_box.setPos(old_matching_group[1])
         else:
             group_box.setPos(CanvasGetNewGroupPos(False))
 
         group_sbox = CanvasBox(group_id, group_name, icon)
+        group_sbox.positionChanged.connect(canvas.qobject.sboxPositionChanged)
+        group_sbox.blockSignals(True)
         group_sbox.setSplit(True, PORT_MODE_INPUT)
 
         group_dict.widgets[1] = group_sbox
 
         if features.handle_group_pos:
             group_sbox.setPos(getStoredCanvasPosition(group_name + "_INPUT", CanvasGetNewGroupPos(True)))
+        elif old_matching_group is not None and old_matching_group[0]:
+            group_sbox.setPos(old_matching_group[2])
         else:
-            group_sbox.setPos(CanvasGetNewGroupPos(True))
+            group_sbox.setPos(group_box.x() + group_box.boundingRect().width() + 300, group_box.y())
 
         canvas.last_z_value += 1
         group_sbox.setZValue(canvas.last_z_value)
@@ -323,29 +378,34 @@ def addGroup(group_id, group_name, split=SPLIT_UNDEF, icon=ICON_APPLICATION):
             CanvasItemFX(group_sbox, True, False)
 
         group_sbox.checkItemPos()
+        group_sbox.blockSignals(False)
 
     else:
         group_box.setSplit(False)
 
         if features.handle_group_pos:
             group_box.setPos(getStoredCanvasPosition(group_name, CanvasGetNewGroupPos(False)))
+        elif old_matching_group is not None:
+            group_box.setPos(old_matching_group[1])
         else:
             # Special ladish fake-split groups
             horizontal = bool(icon == ICON_HARDWARE or icon == ICON_LADISH_ROOM)
             group_box.setPos(CanvasGetNewGroupPos(horizontal))
 
-    group_box.checkItemPos()
-
     canvas.last_z_value += 1
     group_box.setZValue(canvas.last_z_value)
 
+    group_box.checkItemPos()
+    group_box.blockSignals(False)
+
     canvas.group_list.append(group_dict)
 
     if options.eyecandy == EYECANDY_FULL and not options.auto_hide_groups:
         CanvasItemFX(group_box, True, False)
-        return
+    else:
+        QTimer.singleShot(0, canvas.scene.update)
 
-    QTimer.singleShot(0, canvas.scene.update)
+    return group_dict
 
 def removeGroup(group_id):
     if canvas.debug:
@@ -415,6 +475,7 @@ def splitGroup(group_id):
     item = None
     group_name = ""
     group_icon = ICON_APPLICATION
+    group_pos = None
     plugin_id = -1
     plugin_ui = False
     plugin_inline = False
@@ -425,12 +486,14 @@ def splitGroup(group_id):
     for group in canvas.group_list:
         if group.group_id == group_id:
             if group.split:
-                qCritical("PatchCanvas::splitGroup(%i) - group is already split" % group_id)
+                if canvas.debug:
+                    print("PatchCanvas::splitGroup(%i) - group is already split" % group_id)
                 return
 
             item = group.widgets[0]
             group_name = group.group_name
             group_icon = group.icon
+            group_pos = item.pos()
             plugin_id = group.plugin_id
             plugin_ui = group.plugin_ui
             plugin_inline = group.plugin_inline
@@ -443,7 +506,7 @@ def splitGroup(group_id):
     port_list_ids = list(item.getPortList())
 
     for port in canvas.port_list:
-        if port.port_id in port_list_ids:
+        if port.group_id == group_id and port.port_id in port_list_ids:
             port_dict = port_dict_t()
             port_dict.group_id = port.group_id
             port_dict.port_id = port.port_id
@@ -455,7 +518,8 @@ def splitGroup(group_id):
             ports_data.append(port_dict)
 
     for connection in canvas.connection_list:
-        if connection.port_out_id in port_list_ids or connection.port_in_id in port_list_ids:
+        if (connection.group_in_id == group_id and connection.port_in_id in port_list_ids) or \
+           (connection.group_out_id == group_id and connection.port_out_id in port_list_ids):
             connection_dict = connection_dict_t()
             connection_dict.connection_id = connection.connection_id
             connection_dict.group_in_id = connection.group_in_id
@@ -475,7 +539,7 @@ def splitGroup(group_id):
     removeGroup(group_id)
 
     # Step 3 - Re-create Item, now split
-    addGroup(group_id, group_name, SPLIT_YES, group_icon)
+    group = addGroup(group_id, group_name, SPLIT_YES, group_icon)
 
     if plugin_id >= 0:
         setGroupAsPlugin(group_id, plugin_id, plugin_ui, plugin_inline)
@@ -484,7 +548,21 @@ def splitGroup(group_id):
         addPort(group_id, port.port_id, port.port_name, port.port_mode, port.port_type, port.is_alternate)
 
     for conn in conns_data:
-        connectPorts(conn.connection_id, conn.group_out_id, conn.port_out_id, conn.group_in_id, conn.port_in_id)
+        connectPorts(conn.connection_id, conn.group_out_id, conn.port_out_id, conn.group_in_id, conn.port_in_id, True)
+
+    if group is not None:
+        pos1 = group.widgets[0].pos()
+        pos2 = group.widgets[1].pos()
+        group2_pos = QPointF(group_pos.x() + group.widgets[1].boundingRect().width() * 3/2, group_pos.y())
+        group.widgets[0].blockSignals(True)
+        group.widgets[0].setPos(group_pos)
+        group.widgets[0].blockSignals(False)
+        group.widgets[1].blockSignals(True)
+        group.widgets[1].setPos(group2_pos)
+        group.widgets[1].checkItemPos()
+        group.widgets[1].blockSignals(False)
+        valueStr = "%i:%i:%i:%i" % (group_pos.x(), group_pos.y(), group2_pos.x(), group2_pos.y())
+        CanvasCallback(ACTION_GROUP_POSITION, group_id, 0, valueStr)
 
     QTimer.singleShot(0, canvas.scene.update)
 
@@ -496,6 +574,7 @@ def joinGroup(group_id):
     s_item = None
     group_name = ""
     group_icon = ICON_APPLICATION
+    group_pos = None
     plugin_id = -1
     plugin_ui = False
     plugin_inline = False
@@ -506,13 +585,15 @@ def joinGroup(group_id):
     for group in canvas.group_list:
         if group.group_id == group_id:
             if not group.split:
-                qCritical("PatchCanvas::joinGroup(%i) - group is not split" % group_id)
+                if canvas.debug:
+                    print("PatchCanvas::joinGroup(%i) - group is not split" % group_id)
                 return
 
             item = group.widgets[0]
             s_item = group.widgets[1]
             group_name = group.group_name
             group_icon = group.icon
+            group_pos = item.pos()
             plugin_id = group.plugin_id
             plugin_ui = group.plugin_ui
             plugin_inline = group.plugin_inline
@@ -531,7 +612,7 @@ def joinGroup(group_id):
             port_list_ids.append(port_id)
 
     for port in canvas.port_list:
-        if port.port_id in port_list_ids:
+        if port.group_id == group_id and port.port_id in port_list_ids:
             port_dict = port_dict_t()
             port_dict.group_id = port.group_id
             port_dict.port_id = port.port_id
@@ -543,7 +624,8 @@ def joinGroup(group_id):
             ports_data.append(port_dict)
 
     for connection in canvas.connection_list:
-        if connection.port_out_id in port_list_ids or connection.port_in_id in port_list_ids:
+        if (connection.group_in_id == group_id and connection.port_in_id in port_list_ids) or \
+           (connection.group_out_id == group_id and connection.port_out_id in port_list_ids):
             connection_dict = connection_dict_t()
             connection_dict.connection_id = connection.connection_id
             connection_dict.group_in_id = connection.group_in_id
@@ -563,7 +645,7 @@ def joinGroup(group_id):
     removeGroup(group_id)
 
     # Step 3 - Re-create Item, now together
-    addGroup(group_id, group_name, SPLIT_NO, group_icon)
+    group = addGroup(group_id, group_name, SPLIT_NO, group_icon)
 
     if plugin_id >= 0:
         setGroupAsPlugin(group_id, plugin_id, plugin_ui, plugin_inline)
@@ -572,7 +654,15 @@ def joinGroup(group_id):
         addPort(group_id, port.port_id, port.port_name, port.port_mode, port.port_type, port.is_alternate)
 
     for conn in conns_data:
-        connectPorts(conn.connection_id, conn.group_out_id, conn.port_out_id, conn.group_in_id, conn.port_in_id)
+        connectPorts(conn.connection_id, conn.group_out_id, conn.port_out_id, conn.group_in_id, conn.port_in_id, True)
+
+    if group is not None:
+        group.widgets[0].blockSignals(True)
+        group.widgets[0].setPos(group_pos)
+        group.widgets[0].checkItemPos()
+        group.widgets[0].blockSignals(False)
+        valueStr = "%i:%i:%i:%i" % (group_pos.x(), group_pos.y(), 0, 0)
+        CanvasCallback(ACTION_GROUP_POSITION, group_id, 0, valueStr)
 
     QTimer.singleShot(0, canvas.scene.update)
 
@@ -630,10 +720,14 @@ def restoreGroupPositions(dataList):
         if group is None:
             continue
 
+        group.widgets[0].blockSignals(True)
         group.widgets[0].setPos(data['pos1x'], data['pos1y'])
+        group.widgets[0].blockSignals(False)
 
         if group.split and group.widgets[1]:
+            group.widgets[1].blockSignals(True)
             group.widgets[1].setPos(data['pos2x'], data['pos2y'])
+            group.widgets[1].blockSignals(False)
 
 def setGroupPos(group_id, group_pos_x, group_pos_y):
     setGroupPosFull(group_id, group_pos_x, group_pos_y, group_pos_x, group_pos_y)
@@ -645,10 +739,16 @@ def setGroupPosFull(group_id, group_pos_x_o, group_pos_y_o, group_pos_x_i, group
 
     for group in canvas.group_list:
         if group.group_id == group_id:
+            group.widgets[0].blockSignals(True)
             group.widgets[0].setPos(group_pos_x_o, group_pos_y_o)
+            group.widgets[0].checkItemPos()
+            group.widgets[0].blockSignals(False)
 
             if group.split and group.widgets[1]:
+                group.widgets[1].blockSignals(True)
                 group.widgets[1].setPos(group_pos_x_i, group_pos_y_i)
+                group.widgets[1].checkItemPos()
+                group.widgets[1].blockSignals(False)
 
             QTimer.singleShot(0, canvas.scene.update)
             return
@@ -698,6 +798,35 @@ def setGroupAsPlugin(group_id, plugin_id, hasUI, hasInlineDisplay):
 
 # ------------------------------------------------------------------------------------------------------------
 
+def focusGroupUsingPluginId(plugin_id):
+    if canvas.debug:
+        print("PatchCanvas::focusGroupUsingPluginId(%i)" % (plugin_id,))
+
+    if plugin_id < 0 or plugin_id >= MAX_PLUGIN_ID_ALLOWED:
+        return False
+
+    for group in canvas.group_list:
+        if group.plugin_id == plugin_id:
+            item = group.widgets[0]
+            canvas.scene.clearSelection()
+            canvas.scene.getView().centerOn(item)
+            item.setSelected(True)
+            return True
+
+def focusGroupUsingGroupName(group_name):
+    if canvas.debug:
+        print("PatchCanvas::focusGroupUsingGroupName(%s)" % (group_name,))
+
+    for group in canvas.group_list:
+        if group.group_name == group_name:
+            item = group.widgets[0]
+            canvas.scene.clearSelection()
+            canvas.scene.getView().centerOn(item)
+            item.setSelected(True)
+            return True
+
+# ------------------------------------------------------------------------------------------------------------
+
 def addPort(group_id, port_id, port_name, port_mode, port_type, is_alternate=False):
     if canvas.debug:
         print("PatchCanvas::addPort(%i, %i, %s, %s, %s, %s)" % (
@@ -715,7 +844,7 @@ def addPort(group_id, port_id, port_name, port_mode, port_type, is_alternate=Fal
 
     for group in canvas.group_list:
         if group.group_id == group_id:
-            if group.split and group.widgets[0].getSplittedMode() != port_mode and group.widgets[1]:
+            if group.split and group.widgets[0].getSplitMode() != port_mode and group.widgets[1]:
                 n = 1
             else:
                 n = 0
@@ -753,8 +882,13 @@ def removePort(group_id, port_id):
     for port in canvas.port_list:
         if port.group_id == group_id and port.port_id == port_id:
             item = port.widget
-            item.parentItem().removePortFromGroup(port_id)
-            canvas.scene.removeItem(item)
+            try:
+                pitem = item.parentItem()
+                canvas.scene.removeItem(item)
+            except RuntimeError:
+                pass
+            else:
+                pitem.removePortFromGroup(port_id)
             canvas.port_list.remove(port)
             del item
 
@@ -779,7 +913,14 @@ def renamePort(group_id, port_id, new_port_name):
     qCritical("PatchCanvas::renamePort(%i, %i, %s) - Unable to find port to rename" % (
               group_id, port_id, new_port_name.encode()))
 
-def connectPorts(connection_id, group_out_id, port_out_id, group_in_id, port_in_id):
+def connectPorts(connection_id, group_out_id, port_out_id, group_in_id, port_in_id, fromSplitOrJoin = False):
+    if canvas.last_connection_id >= connection_id and not fromSplitOrJoin:
+        print("PatchCanvas::connectPorts(%i, %i, %i, %i, %i) - invalid connection id received (last: %i)" % (
+              connection_id, group_out_id, port_out_id, group_in_id, port_in_id, canvas.last_connection_id))
+        return
+
+    canvas.last_connection_id = connection_id
+
     if canvas.debug:
         print("PatchCanvas::connectPorts(%i, %i, %i, %i, %i)" % (
               connection_id, group_out_id, port_out_id, group_in_id, port_in_id))
@@ -878,8 +1019,12 @@ def disconnectPorts(connection_id):
         qCritical("PatchCanvas::disconnectPorts(%i) - unable to find input port" % connection_id)
         return
 
-    item1.parentItem().removeLineFromGroup(connection_id)
-    item2.parentItem().removeLineFromGroup(connection_id)
+    item1p = item1.parentItem()
+    item2p = item2.parentItem()
+    if item1p:
+        item1p.removeLineFromGroup(connection_id)
+    if item2p:
+        item2p.removeLineFromGroup(connection_id)
 
     if options.eyecandy == EYECANDY_FULL:
         CanvasItemFX(line, False, True)
@@ -926,6 +1071,8 @@ def handlePluginRemoved(plugin_id):
     if canvas.debug:
         print("PatchCanvas::handlePluginRemoved(%i)" % plugin_id)
 
+    canvas.scene.clearSelection()
+
     group = canvas.group_plugin_map.pop(plugin_id, None)
 
     if group is not None:
