@@ -9,7 +9,7 @@
 //
 //-----------------------------------------------------------------------------
 // LICENSE
-// (c) 2019, Steinberg Media Technologies GmbH, All Rights Reserved
+// (c) 2021, Steinberg Media Technologies GmbH, All Rights Reserved
 //-----------------------------------------------------------------------------
 // Redistribution and use in source and binary forms, with or without modification,
 // are permitted provided that the following conditions are met:
@@ -48,6 +48,7 @@
 
 #if SMTG_OS_WINDOWS
 #include <windows.h>
+#ifdef _MSC_VER
 #pragma warning (disable : 4244)
 #pragma warning (disable : 4267)
 #pragma warning (disable : 4996)
@@ -59,9 +60,9 @@
 #define realloc(p,s) _realloc_dbg(p,s,  _NORMAL_BLOCK, __FILE__, __LINE__)
 #define free(p) _free_dbg(p, _NORMAL_BLOCK)
 
-#endif
-
-#endif
+#endif // DEVELOPMENT
+#endif // _MSC_VER
+#endif // SMTG_OS_WINDOWS
 
 #ifndef kPrintfBufferSize
 #define kPrintfBufferSize 4096
@@ -175,7 +176,7 @@ static CFStringDebugAllocator gDebugAllocator;
 #else
 
 static const CFAllocatorRef kCFAllocator = ::kCFAllocatorDefault;
-#endif
+#endif // SMTG_DEBUG_CFALLOCATOR
 }
 
 //-----------------------------------------------------------------------------
@@ -327,7 +328,7 @@ static inline Steinberg::int32 strnicmp16 (const Steinberg::char16* str1, const
 
 	CFIndex str1Len = Steinberg::strlen16 (str1);
 	CFIndex str2Len = Steinberg::strlen16 (str2);
-	if (size < str2Len) // range is not applied to second string
+	if (static_cast<CFIndex> (size) < str2Len) // range is not applied to second string
 		str2Len = size;
 	CFStringRef cfStr1 = CFStringCreateWithCharactersNoCopy (Steinberg::kCFAllocator, (UniChar*)str1, str1Len, kCFAllocatorNull);
 	CFStringRef cfStr2 = CFStringCreateWithCharactersNoCopy (Steinberg::kCFAllocator, (UniChar*)str2, str2Len, kCFAllocatorNull);
@@ -414,7 +415,7 @@ static inline Steinberg::int32 sprintf16 (Steinberg::char16* str, const Steinber
 	return vsnwprintf (str, -1, format, marker);
 }
 
-#endif
+#endif // SMTG_OS_LINUX
 
 /*
 UTF-8                EF BB BF 
@@ -467,7 +468,7 @@ ConstString::ConstString (const ConstString& str, int32 offset, int32 length)
 
 //-----------------------------------------------------------------------------
 ConstString::ConstString (const FVariant& var)
-: buffer (0)
+: buffer (nullptr)
 , len (0)
 , isWide (0) 
 {
@@ -489,7 +490,7 @@ ConstString::ConstString (const FVariant& var)
 
 //-----------------------------------------------------------------------------
 ConstString::ConstString ()
-: buffer (0)
+: buffer (nullptr)
 , len (0)
 , isWide (0) 
 {
@@ -1629,7 +1630,7 @@ char8 ConstString::toLower (char8 c)
         ::CharLowerA (temp);
         return temp[0];
 	#else
-		return tolower (c);
+		return static_cast<char8> (tolower (c));
 	#endif
 }
 
@@ -1643,7 +1644,7 @@ char8 ConstString::toUpper (char8 c)
         ::CharUpperA (temp);
         return temp[0];
 	#else
-		return toupper (c);
+		return static_cast<char8> (toupper (c));
 	#endif
 }
 
@@ -1856,7 +1857,7 @@ static CFStringEncoding MBCodePageToCFStringEncoding (uint32 codePage)
 //-----------------------------------------------------------------------------
 int32 ConstString::multiByteToWideString (char16* dest, const char8* source, int32 charCount, uint32 sourceCodePage)
 {
-	if (source == 0 || source[0] == 0)
+	if (source == nullptr || source[0] == 0)
 	{
 		if (dest && charCount > 0)
 		{
@@ -1889,7 +1890,7 @@ int32 ConstString::multiByteToWideString (char16* dest, const char8* source, int
 #endif
 
 #if SMTG_OS_LINUX
-	if (sourceCodePage == kCP_ANSI || sourceCodePage == kCP_Utf8)
+	if (sourceCodePage == kCP_ANSI || sourceCodePage == kCP_US_ASCII || sourceCodePage == kCP_Utf8)
 	{
 		if (dest == nullptr)
 		{
@@ -1909,7 +1910,7 @@ int32 ConstString::multiByteToWideString (char16* dest, const char8* source, int
 		}
 	}
 	else
-    {
+	{
 		assert(false && "DEPRECATED No Linux implementation");
 	}
 
@@ -1923,7 +1924,7 @@ int32 ConstString::multiByteToWideString (char16* dest, const char8* source, int
 int32 ConstString::wideStringToMultiByte (char8* dest, const char16* wideString, int32 charCount, uint32 destCodePage)
 {
 #if SMTG_OS_WINDOWS
-	return WideCharToMultiByte (destCodePage, 0, wideString, -1, dest, charCount, 0, 0);
+	return WideCharToMultiByte (destCodePage, 0, wideString, -1, dest, charCount, nullptr, nullptr);
 
 #elif SMTG_OS_MACOS
 	int32 result = 0;
@@ -1966,7 +1967,7 @@ int32 ConstString::wideStringToMultiByte (char8* dest, const char16* wideString,
 			}
 		}
 	}
-	else if (destCodePage == kCP_ANSI)
+	else if (destCodePage == kCP_ANSI || destCodePage == kCP_US_ASCII)
 	{
 		if (dest == nullptr)
 		{
@@ -1989,7 +1990,7 @@ int32 ConstString::wideStringToMultiByte (char8* dest, const char16* wideString,
 		}
 	}
 	else
-    {
+	{
 		assert(false && "DEPRECATED No Linux implementation");
 	}
 	return result;
@@ -2012,7 +2013,7 @@ bool ConstString::isNormalized (UnicodeNormalization n)
 #ifdef UNICODE
 	if (n != kUnicodeNormC)
 		return false;
-	uint32 normCharCount = static_cast<uint32> (FoldString (MAP_PRECOMPOSED, buffer16, len, 0, 0));
+	uint32 normCharCount = static_cast<uint32> (FoldString (MAP_PRECOMPOSED, buffer16, len, nullptr, 0));
 	return (normCharCount == len);
 #else
 	return false; 
@@ -2115,7 +2116,7 @@ String::String (String&& str)
 //-----------------------------------------------------------------------------
 String& String::operator= (String&& str)
 {
-	SMTG_ASSERT (buffer == 0 || buffer != str.buffer);
+	SMTG_ASSERT (buffer == nullptr || buffer != str.buffer);
 	tryFreeBuffer ();
 	
 	isWide = str.isWide;
@@ -2143,7 +2144,7 @@ bool String::toWideString (uint32 sourceCodePage)
 	{
 		if (buffer8 && len > 0)
 		{
-			int32 bytesNeeded = multiByteToWideString (0, buffer8, 0, sourceCodePage) * sizeof (char16);
+			int32 bytesNeeded = multiByteToWideString (nullptr, buffer8, 0, sourceCodePage) * sizeof (char16);
 			if (bytesNeeded)
 			{
 				bytesNeeded += sizeof (char16);
@@ -2227,7 +2228,7 @@ bool String::toMultiByte (uint32 destCodePage)
 	{
 		if (buffer16 && len > 0)
 		{
-			int32 numChars = wideStringToMultiByte (0, buffer16, 0, destCodePage) + sizeof (char8);
+			int32 numChars = wideStringToMultiByte (nullptr, buffer16, 0, destCodePage) + sizeof (char8);
 			char8* newStr = (char8*) malloc (numChars * sizeof (char8));
 			if (wideStringToMultiByte (newStr, buffer16, numChars, destCodePage) <= 0)
 			{
@@ -2263,7 +2264,7 @@ bool String::normalize (UnicodeNormalization n)
 	if (isWide == false)
 		return false;
 
-	if (buffer16 == 0)
+	if (buffer16 == nullptr)
 		return true;
 
 #if SMTG_OS_WINDOWS
@@ -2271,7 +2272,7 @@ bool String::normalize (UnicodeNormalization n)
 	if (n != kUnicodeNormC)
 		return false;
 
-	uint32 normCharCount = static_cast<uint32> (FoldString (MAP_PRECOMPOSED, buffer16, len, 0, 0));
+	uint32 normCharCount = static_cast<uint32> (FoldString (MAP_PRECOMPOSED, buffer16, len, nullptr, 0));
 	if (normCharCount == len)
 		return true;
 
@@ -2320,7 +2321,7 @@ void String::tryFreeBuffer ()
 	if (buffer)
 	{
 		free (buffer);
-		buffer = 0;
+		buffer = nullptr;
 	}
 }
 
@@ -2348,7 +2349,7 @@ bool String::resize (uint32 newLength, bool wide, bool fill)
 			if (newBufferSize != oldBufferSize)
 			{
 				void* newstr = realloc (buffer, newBufferSize);
-				if (newstr == 0)
+				if (newstr == nullptr)
 					return false;
 				buffer = newstr;
 				if (isWide)
@@ -2362,7 +2363,7 @@ bool String::resize (uint32 newLength, bool wide, bool fill)
 		else
 		{
 			void* newstr = malloc (newBufferSize);
-			if (newstr == 0)
+			if (newstr == nullptr)
 				return false;
 			buffer = newstr;
 			if (isWide)
@@ -2520,7 +2521,7 @@ String& String::assign (const char8* str, int32 n, bool isTerminated)
 
 	if (resize (n, false))
 	{
-		if (buffer8 && n > 0)
+		if (buffer8 && n > 0 && str)
 		{
 			memcpy (buffer8, str, n * sizeof (char8));
 			SMTG_ASSERT (buffer8[n] == 0)
@@ -2547,7 +2548,7 @@ String& String::assign (const char16* str, int32 n, bool isTerminated)
 
 	if (resize (n, true))
 	{
-		if (buffer16 && n > 0)
+		if (buffer16 && n > 0 && str)
 		{
 			memcpy (buffer16, str, n * sizeof (char16));
 			SMTG_ASSERT (buffer16[n] == 0)
@@ -2628,7 +2629,7 @@ String& String::append (const char8* str, int32 n)
 		if (!resize (newlen, false))
 			return *this;
 
-		if (buffer)
+		if (buffer && str)
 		{
 			memcpy (buffer8 + len, str, n * sizeof (char8));
 			SMTG_ASSERT (buffer8[newlen] == 0)
@@ -2663,7 +2664,7 @@ String& String::append (const char16* str, int32 n)
 		if (!resize (newlen, true))
 			return *this;
 
-		if (buffer16)
+		if (buffer16 && str)
 		{
 			memcpy (buffer16 + len, str, n * sizeof (char16));
 			SMTG_ASSERT (buffer16[newlen] == 0)
@@ -2772,7 +2773,7 @@ String& String::insertAt (uint32 idx, const char8* str, int32 n)
 		if (!resize (newlen, false))
 			return *this;
 
-		if (buffer)
+		if (buffer && str)
 		{
 			if (idx < len)
 				memmove (buffer8 + idx + n, buffer8 + idx, (len - idx) * sizeof (char8));
@@ -2806,7 +2807,7 @@ String& String::insertAt (uint32 idx, const char16* str, int32 n)
 		if (!resize (newlen, true))
 			return *this;
 
-		if (buffer)
+		if (buffer && str)
 		{
 			if (idx < len)
 				memmove (buffer16 + idx + n, buffer16 + idx, (len - idx) * sizeof (char16));
@@ -2833,7 +2834,7 @@ String& String::replace (uint32 idx, int32 n1, const ConstString& str, int32 n2)
 //-----------------------------------------------------------------------------
 String& String::replace (uint32 idx, int32 n1, const char8* str, int32 n2)
 {
-	if (idx > len || str == 0)
+	if (idx > len || str == nullptr)
 		return *this;
 
 	if (isWide)
@@ -2874,7 +2875,7 @@ String& String::replace (uint32 idx, int32 n1, const char8* str, int32 n2)
 //-----------------------------------------------------------------------------
 String& String::replace (uint32 idx, int32 n1, const char16* str, int32 n2)
 {
-	if (idx > len || str == 0)
+	if (idx > len || str == nullptr)
 		return *this;
 
 	if (!isWide)
@@ -2911,7 +2912,7 @@ String& String::replace (uint32 idx, int32 n1, const char16* str, int32 n2)
 //-----------------------------------------------------------------------------
 int32 String::replace (const char8* toReplace, const char8* toReplaceWith, bool all, CompareMode m)
 {
-	if (toReplace == 0 || toReplaceWith == 0)
+	if (toReplace == nullptr || toReplaceWith == nullptr)
 		return 0;
 
 	int32 result = 0;
@@ -2939,7 +2940,7 @@ int32 String::replace (const char8* toReplace, const char8* toReplaceWith, bool
 //-----------------------------------------------------------------------------
 int32 String::replace (const char16* toReplace, const char16* toReplaceWith, bool all, CompareMode m)
 {
-	if (toReplace == 0 || toReplaceWith == 0)
+	if (toReplace == nullptr || toReplaceWith == nullptr)
 		return 0;
 
 	int32 result = 0;
@@ -3264,7 +3265,7 @@ static uint32 performRemoveChars (T* str, uint32 length, const T* toRemove)
 //-----------------------------------------------------------------------------
 bool String::removeChars8 (const char8* toRemove)
 {
-	if (isEmpty () || toRemove == 0)
+	if (isEmpty () || toRemove == nullptr)
 		return true;
 
 	if (isWide)
@@ -3288,7 +3289,7 @@ bool String::removeChars8 (const char8* toRemove)
 //-----------------------------------------------------------------------------
 bool String::removeChars16 (const char16* toRemove)
 {
-	if (isEmpty () || toRemove == 0)
+	if (isEmpty () || toRemove == nullptr)
 		return true;
 
 	if (!isWide)
@@ -3649,7 +3650,7 @@ void String::take (String& other)
 	buffer = other.buffer;
 	len = other.len;
 
-	other.buffer = 0;
+	other.buffer = nullptr;
 	other.len = 0;
 }
 
@@ -3667,7 +3668,7 @@ void* String::pass ()
 {
 	void* res = buffer;
 	len = 0;
-	buffer = 0;
+	buffer = nullptr;
 	return res;
 }
 
@@ -3857,11 +3858,11 @@ uint32 hashString16 (const char16* s, uint32 m)
 //------------------------------------------------------------------------
 template <class T> int32 tstrnatcmp (const T* s1, const T* s2, bool caseSensitive = true)
 {
-	if (s1 == 0 && s2 == 0)
+	if (s1 == nullptr && s2 == nullptr)
 		return 0;
-	else if (s1 == 0)
+	else if (s1 == nullptr)
 		return -1;
-	else if (s2 == 0)
+	else if (s2 == nullptr)
 		return 1;
 
 	while (*s1 && *s2)
@@ -3907,8 +3908,8 @@ template <class T> int32 tstrnatcmp (const T* s1, const T* s2, bool caseSensitiv
 		{
 			if (caseSensitive == false)
 			{
-				T srcToUpper = toupper (*s1);
-				T dstToUpper = toupper (*s2);
+				T srcToUpper = static_cast<T> (toupper (*s1));
+				T dstToUpper = static_cast<T> (toupper (*s2));
 				if (srcToUpper != dstToUpper)
 					return (int32)(srcToUpper - dstToUpper);
 			}
