@@ -1,6 +1,6 @@
 /*
  * Carla JACK API for external applications
- * Copyright (C) 2016-2019 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2016-2022 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,7 +21,7 @@ CARLA_BACKEND_USE_NAMESPACE
 
 // --------------------------------------------------------------------------------------------------------------------
 
-CARLA_EXPORT
+CARLA_PLUGIN_EXPORT
 jack_nframes_t jack_midi_get_event_count(void* buf)
 {
     const JackMidiPortBufferBase* const jbasebuf((const JackMidiPortBufferBase*)buf);
@@ -36,7 +36,7 @@ jack_nframes_t jack_midi_get_event_count(void* buf)
     return jmidibuf->count;
 }
 
-CARLA_EXPORT
+CARLA_PLUGIN_EXPORT
 int jack_midi_event_get(jack_midi_event_t* ev, void* buf, uint32_t index)
 {
     const JackMidiPortBufferBase* const jbasebuf((const JackMidiPortBufferBase*)buf);
@@ -53,7 +53,7 @@ int jack_midi_event_get(jack_midi_event_t* ev, void* buf, uint32_t index)
     return 0;
 }
 
-CARLA_EXPORT
+CARLA_PLUGIN_EXPORT
 void jack_midi_clear_buffer(void* buf)
 {
     JackMidiPortBufferBase* const jbasebuf((JackMidiPortBufferBase*)buf);
@@ -70,19 +70,19 @@ void jack_midi_clear_buffer(void* buf)
     std::memset(jmidibuf->bufferPool, 0, JackMidiPortBufferBase::kBufferPoolSize);
 }
 
-CARLA_EXPORT
+CARLA_PLUGIN_EXPORT
 void jack_midi_reset_buffer(void* buf)
 {
     jack_midi_clear_buffer(buf);
 }
 
-CARLA_EXPORT
+CARLA_PLUGIN_EXPORT
 size_t jack_midi_max_event_size(void*)
 {
     return JackMidiPortBufferBase::kMaxEventSize;
 }
 
-CARLA_EXPORT
+CARLA_PLUGIN_EXPORT
 jack_midi_data_t* jack_midi_event_reserve(void* buf, jack_nframes_t frame, size_t size)
 {
     JackMidiPortBufferBase* const jbasebuf((JackMidiPortBufferBase*)buf);
@@ -112,7 +112,7 @@ jack_midi_data_t* jack_midi_event_reserve(void* buf, jack_nframes_t frame, size_
     return jmdata;
 }
 
-CARLA_EXPORT
+CARLA_PLUGIN_EXPORT
 int jack_midi_event_write(void* buf, jack_nframes_t frame, const jack_midi_data_t* data, size_t size)
 {
     JackMidiPortBufferBase* const jbasebuf((JackMidiPortBufferBase*)buf);
@@ -142,7 +142,7 @@ int jack_midi_event_write(void* buf, jack_nframes_t frame, const jack_midi_data_
     return 0;
 }
 
-CARLA_EXPORT
+CARLA_PLUGIN_EXPORT
 uint32_t jack_midi_get_lost_event_count(void*)
 {
     return 0;
