@@ -1,6 +1,6 @@
 /*
  * Carla Plugin Host
- * Copyright (C) 2011-2020 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2011-2022 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -49,6 +49,7 @@
 #define JUCE_MODULE_AVAILABLE_juce_events                1
 #define JUCE_MODULE_AVAILABLE_juce_graphics              1
 #define JUCE_MODULE_AVAILABLE_juce_gui_basics            1
+#define JUCE_MODULE_AVAILABLE_juce_gui_extra             1
 
 // always disabled
 #define JUCE_MODULE_AVAILABLE_juce_audio_formats         0
@@ -65,12 +66,6 @@
 # define JUCE_MODULE_AVAILABLE_juce_audio_devices        0
 #endif
 
-#ifdef USING_JUCE_GUI_EXTRA
-# define JUCE_MODULE_AVAILABLE_juce_gui_extra            1
-#else
-# define JUCE_MODULE_AVAILABLE_juce_gui_extra            0
-#endif
-
 // misc
 #define JUCE_DISABLE_JUCE_VERSION_PRINTING 1
 #define JUCE_GLOBAL_MODULE_SETTINGS_INCLUDED 1
@@ -208,6 +203,8 @@
 // --------------------------------------------------------------------------------------------------------------------
 // juce_audio_processors
 
+#define JUCE_VST_FALLBACK_HOST_NAME "Carla"
+
 //=============================================================================
 /** Config: JUCE_PLUGINHOST_VST
     Enables the VST audio plugin hosting classes. This requires the Steinberg VST SDK to be
@@ -227,7 +224,7 @@
 
     @see VSTPluginFormat, VST3PluginFormat, AudioPluginFormat, AudioPluginFormatManager, JUCE_PLUGINHOST_VST, JUCE_PLUGINHOST_AU
 */
-#if defined(APPCONFIG_OS_LINUX) || defined(APPCONFIG_OS_FREEBSD) || defined(APPCONFIG_OS_MAC) || defined(APPCONFIG_OS_WIN)
+#if defined(APPCONFIG_OS_LINUX) || defined(APPCONFIG_OS_MAC) || defined(APPCONFIG_OS_WIN)
 # define JUCE_PLUGINHOST_VST3 1
 #else
 # define JUCE_PLUGINHOST_VST3 0
@@ -357,11 +354,7 @@
 
 #define JUCE_INCLUDE_JPEGLIB_CODE 1
 
-#ifdef APPCONFIG_OS_MAC
-# define USE_COREGRAPHICS_RENDERING 1
-#else
-# define USE_COREGRAPHICS_RENDERING 0
-#endif
+#define USE_COREGRAPHICS_RENDERING 0
 
 // --------------------------------------------------------------------------------------------------------------------
 // juce_gui_basics
