@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -349,13 +349,9 @@ public:
         Array<double> newRates;
 
         if (asioObject != nullptr)
-        {
-            for (auto rate : { 8000, 11025, 16000, 22050, 32000,
-                               44100, 48000, 88200, 96000, 176400,
-                               192000, 352800, 384000, 705600, 768000 })
-                if (asioObject->canSampleRate ((double) rate) == 0)
-                    newRates.add ((double) rate);
-        }
+            for (const auto rate : SampleRateHelpers::getAllSampleRates())
+                if (asioObject->canSampleRate (rate) == 0)
+                    newRates.add (rate);
 
         if (newRates.isEmpty())
         {
@@ -399,7 +395,7 @@ public:
 
         jassert (currentCallback == nullptr);
 
-        if (bufferSizeSamples < 8 || bufferSizeSamples > 16384)
+        if (bufferSizeSamples < 8 || bufferSizeSamples > 32768)
             shouldUsePreferredSize = true;
 
         if (asioObject == nullptr)
@@ -415,11 +411,8 @@ public:
         auto err = asioObject->getChannels (&totalNumInputChans, &totalNumOutputChans);
         jassert (err == ASE_OK);
 
-        bufferSizeSamples = readBufferSizes (bufferSizeSamples);
-
         auto sampleRate = sr;
         currentSampleRate = sampleRate;
-        currentBlockSizeSamples = bufferSizeSamples;
         currentChansOut.clear();
         currentChansIn.clear();
 
@@ -441,6 +434,7 @@ public:
         buffersCreated = false;
 
         setSampleRate (sampleRate);
+        currentBlockSizeSamples = bufferSizeSamples = readBufferSizes (bufferSizeSamples);
 
         // (need to get this again in case a sample rate change affected the channel count)
         err = asioObject->getChannels (&totalNumInputChans, &totalNumOutputChans);
@@ -821,7 +815,15 @@ private:
 
     long refreshBufferSizes()
     {
-        return asioObject->getBufferSize (&minBufferSize, &maxBufferSize, &preferredBufferSize, &bufferGranularity);
+        const auto err = asioObject->getBufferSize (&minBufferSize, &maxBufferSize, &preferredBufferSize, &bufferGranularity);
+
+        if (err == ASE_OK)
+        {
+            bufferSizes.clear();
+            addBufferSizes (minBufferSize, maxBufferSize, preferredBufferSize, bufferGranularity);
+        }
+
+        return err;
     }
 
     int readBufferSizes (int bufferSizeSamples)
@@ -913,7 +915,7 @@ private:
         {
             granularity = jmax (16, (int) granularity);
 
-            for (int i = jmax ((int) (minSize + 15) & ~15, (int) granularity); i < jmin (6400, (int) maxSize); i += granularity)
+            for (int i = jmax ((int) (minSize + 15) & ~15, (int) granularity); i <= jmin (6400, (int) maxSize); i += granularity)
                 bufferSizes.addIfNotAlreadyThere (granularity * (i / granularity));
         }
         else if (granularity < 0)
@@ -945,15 +947,18 @@ private:
         {
             JUCE_ASIO_LOG ("rate change: " + String (currentSampleRate) + " to " + String (newRate));
             auto err = asioObject->setSampleRate (newRate);
+            JUCE_ASIO_LOG_ERROR ("setSampleRate", err);
+            Thread::sleep (10);
 
             if (err == ASE_NoClock && numClockSources > 0)
             {
                 JUCE_ASIO_LOG ("trying to set a clock source..");
-                Thread::sleep (10);
                 err = asioObject->setClockSource (clocks[0].index);
                 JUCE_ASIO_LOG_ERROR ("setClockSource2", err);
                 Thread::sleep (10);
                 err = asioObject->setSampleRate (newRate);
+                JUCE_ASIO_LOG_ERROR ("setSampleRate", err);
+                Thread::sleep (10);
             }
 
             if (err == 0)
@@ -1214,8 +1219,6 @@ private:
 
                     if ((err = refreshBufferSizes()) == 0)
                     {
-                        addBufferSizes (minBufferSize, maxBufferSize, preferredBufferSize, bufferGranularity);
-
                         auto currentRate = getSampleRate();
 
                         if (currentRate < 1.0 || currentRate > 192001.0)
@@ -1323,8 +1326,12 @@ private:
                     inputFormat[i].convertToFloat (infos[i].buffers[bufferIndex], inBuffers[i], samps);
                 }
 
-                currentCallback->audioDeviceIOCallback (const_cast<const float**> (inBuffers.getData()), numActiveInputChans,
-                                                        outBuffers, numActiveOutputChans, samps);
+                currentCallback->audioDeviceIOCallbackWithContext (const_cast<const float**> (inBuffers.getData()),
+                                                                   numActiveInputChans,
+                                                                   outBuffers,
+                                                                   numActiveOutputChans,
+                                                                   samps,
+                                                                   {});
 
                 for (int i = 0; i < numActiveOutputChans; ++i)
                 {
