@@ -2,15 +2,15 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
-   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-6-licence
+   End User License Agreement: www.juce.com/juce-7-licence
    Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
@@ -31,58 +31,75 @@ static int numAlwaysOnTopPeers = 0;
 bool juce_areThereAnyAlwaysOnTopWindows()  { return numAlwaysOnTopPeers > 0; }
 
 //==============================================================================
-template<typename WindowHandleType>
-class LinuxComponentPeer  : public ComponentPeer
+class LinuxComponentPeer  : public ComponentPeer,
+                            private XWindowSystemUtilities::XSettings::Listener
 {
 public:
-    LinuxComponentPeer (Component& comp, int windowStyleFlags, WindowHandleType parentToAddTo)
+    LinuxComponentPeer (Component& comp, int windowStyleFlags, ::Window parentToAddTo)
         : ComponentPeer (comp, windowStyleFlags),
           isAlwaysOnTop (comp.isAlwaysOnTop())
     {
-        // it's dangerous to create a window on a thread other than the message thread..
+        // it's dangerous to create a window on a thread other than the message thread.
         JUCE_ASSERT_MESSAGE_MANAGER_IS_LOCKED
 
+        const auto* instance = XWindowSystem::getInstance();
+
+        if (! instance->isX11Available())
+            return;
+
         if (isAlwaysOnTop)
             ++numAlwaysOnTopPeers;
 
         repainter = std::make_unique<LinuxRepaintManager> (*this);
 
-        windowH = XWindowSystem::getInstance()->createWindow (parentToAddTo, this);
+        windowH = instance->createWindow (parentToAddTo, this);
         parentWindow = parentToAddTo;
 
         setTitle (component.getName());
 
+        if (auto* xSettings = instance->getXSettings())
+            xSettings->addListener (this);
+
         getNativeRealtimeModifiers = []() -> ModifierKeys { return XWindowSystem::getInstance()->getNativeRealtimeModifiers(); };
     }
 
     ~LinuxComponentPeer() override
     {
-        // it's dangerous to delete a window on a thread other than the message thread..
+        // it's dangerous to delete a window on a thread other than the message thread.
         JUCE_ASSERT_MESSAGE_MANAGER_IS_LOCKED
 
+        auto* instance = XWindowSystem::getInstance();
+
         repainter = nullptr;
-        XWindowSystem::getInstance()->destroyWindow (windowH);
+        instance->destroyWindow (windowH);
+
+        if (auto* xSettings = instance->getXSettings())
+            xSettings->removeListener (this);
 
         if (isAlwaysOnTop)
             --numAlwaysOnTopPeers;
     }
 
-    //==============================================================================
-    void* getNativeHandle() const override
+    ::Window getWindowHandle() const noexcept
     {
-        return (void*) windowH;
+        return windowH;
     }
 
     //==============================================================================
-    void setBounds (const Rectangle<int>& newBounds, bool isNowFullScreen) override
+    void* getNativeHandle() const override
     {
-        bounds = newBounds.withSize (jmax (1, newBounds.getWidth()),
-                                     jmax (1, newBounds.getHeight()));
+        return reinterpret_cast<void*> (getWindowHandle());
+    }
+
+    //==============================================================================
+    void forceSetBounds (const Rectangle<int>& correctedNewBounds, bool isNowFullScreen)
+    {
+        bounds = correctedNewBounds;
 
         updateScaleFactorFromNewBounds (bounds, false);
 
-        auto physicalBounds = (parentWindow == 0 ? Desktop::getInstance().getDisplays().logicalToPhysical (bounds)
-                               : bounds * currentScaleFactor);
+        auto physicalBounds = parentWindow == 0 ? Desktop::getInstance().getDisplays().logicalToPhysical (bounds)
+                                                : bounds * currentScaleFactor;
 
         WeakReference<Component> deletionChecker (&component);
 
@@ -97,15 +114,27 @@ public:
         }
     }
 
+    void setBounds (const Rectangle<int>& newBounds, bool isNowFullScreen) override
+    {
+        const auto correctedNewBounds = newBounds.withSize (jmax (1, newBounds.getWidth()),
+                                                            jmax (1, newBounds.getHeight()));
+
+        if (bounds != correctedNewBounds || fullScreen != isNowFullScreen)
+            forceSetBounds (correctedNewBounds, isNowFullScreen);
+    }
+
     Point<int> getScreenPosition (bool physical) const
     {
-        auto parentPosition = XWindowSystem::getInstance()->getParentScreenPosition();
+        auto physicalParentPosition = XWindowSystem::getInstance()->getPhysicalParentScreenPosition();
+        auto parentPosition = parentWindow == 0 ? Desktop::getInstance().getDisplays().physicalToLogical (physicalParentPosition)
+                                                : physicalParentPosition / currentScaleFactor;
 
-        auto screenBounds = (parentWindow == 0 ? bounds
-                                               : bounds.translated (parentPosition.x, parentPosition.y));
+        auto screenBounds = parentWindow == 0 ? bounds
+                                              : bounds.translated (parentPosition.x, parentPosition.y);
 
         if (physical)
-            return Desktop::getInstance().getDisplays().logicalToPhysical (screenBounds.getTopLeft());
+            return parentWindow == 0 ? Desktop::getInstance().getDisplays().logicalToPhysical (screenBounds.getTopLeft())
+                                     : screenBounds.getTopLeft() * currentScaleFactor;
 
         return screenBounds.getTopLeft();
     }
@@ -115,19 +144,25 @@ public:
         return bounds;
     }
 
-    BorderSize<int> getFrameSize() const override
+    OptionalBorderSize getFrameSizeIfPresent() const override
     {
         return windowBorder;
     }
 
+    BorderSize<int> getFrameSize() const override
+    {
+        const auto optionalBorderSize = getFrameSizeIfPresent();
+        return optionalBorderSize ? (*optionalBorderSize) : BorderSize<int>();
+    }
+
     Point<float> localToGlobal (Point<float> relativePosition) override
     {
-        return relativePosition + getScreenPosition (false).toFloat();
+        return localToGlobal (*this, relativePosition);
     }
 
     Point<float> globalToLocal (Point<float> screenPosition) override
     {
-        return screenPosition - getScreenPosition (false).toFloat();
+        return globalToLocal (*this, screenPosition);
     }
 
     using ComponentPeer::localToGlobal;
@@ -170,8 +205,14 @@ public:
 
         if (fullScreen != shouldBeFullScreen)
         {
+            const auto usingNativeTitleBar = ((styleFlags & windowHasTitleBar) != 0);
+
+            if (usingNativeTitleBar)
+                XWindowSystem::getInstance()->setMaximised (windowH, shouldBeFullScreen);
+
             if (shouldBeFullScreen)
-                r = Desktop::getInstance().getDisplays().getMainDisplay().userArea;
+                r = usingNativeTitleBar ? XWindowSystem::getInstance()->getWindowBounds (windowH, parentWindow)
+                                        : Desktop::getInstance().getDisplays().getDisplayForRect (bounds)->userArea;
 
             if (! r.isEmpty())
                 setBounds (ScalingHelpers::scaledScreenPosToUnscaled (component, r), shouldBeFullScreen);
@@ -200,8 +241,11 @@ public:
             if (! c->isVisible())
                 continue;
 
-            if (auto* peer = c->getPeer())
-                if (peer->contains (localPos + bounds.getPosition() - peer->getBounds().getPosition(), true))
+            auto* otherPeer = c->getPeer();
+            jassert (otherPeer == nullptr || dynamic_cast<LinuxComponentPeer*> (c->getPeer()) != nullptr);
+
+            if (auto* peer = static_cast<LinuxComponentPeer*> (otherPeer))
+                if (peer->contains (globalToLocal (*peer, localToGlobal (*this, localPos.toFloat())).roundToInt(), true))
                     return false;
         }
 
@@ -253,12 +297,14 @@ public:
     //==============================================================================
     void repaint (const Rectangle<int>& area) override
     {
-        repainter->repaint (area.getIntersection (bounds.withZeroOrigin()));
+        if (repainter != nullptr)
+            repainter->repaint (area.getIntersection (bounds.withZeroOrigin()));
     }
 
     void performAnyPendingRepaintsNow() override
     {
-        repainter->performAnyPendingRepaintsNow();
+        if (repainter != nullptr)
+            repainter->performAnyPendingRepaintsNow();
     }
 
     void setIcon (const Image& newIcon) override
@@ -295,30 +341,53 @@ public:
     }
 
     //==============================================================================
-    WindowHandleType getParentWindow()                 { return parentWindow; }
-    void setParentWindow (WindowHandleType newParent)  { parentWindow = newParent; }
+    ::Window getParentWindow()                         { return parentWindow; }
+    void setParentWindow (::Window newParent)          { parentWindow = newParent; }
 
     //==============================================================================
+    bool isConstrainedNativeWindow() const
+    {
+        return constrainer != nullptr
+            && (styleFlags & (windowHasTitleBar | windowIsResizable)) == (windowHasTitleBar | windowIsResizable)
+            && ! isKioskMode();
+    }
+
     void updateWindowBounds()
     {
-        jassert (windowH != 0);
-        if (windowH != 0)
+        if (windowH == 0)
         {
-            auto physicalBounds = XWindowSystem::getInstance()->getWindowBounds (windowH, parentWindow);
-
-            updateScaleFactorFromNewBounds (physicalBounds, true);
-
-            bounds = (parentWindow == 0 ? Desktop::getInstance().getDisplays().physicalToLogical (physicalBounds)
-                                        : physicalBounds / currentScaleFactor);
+            jassertfalse;
+            return;
         }
+
+        if (isConstrainedNativeWindow())
+            XWindowSystem::getInstance()->updateConstraints (windowH);
+
+        auto physicalBounds = XWindowSystem::getInstance()->getWindowBounds (windowH, parentWindow);
+
+        updateScaleFactorFromNewBounds (physicalBounds, true);
+
+        bounds = parentWindow == 0 ? Desktop::getInstance().getDisplays().physicalToLogical (physicalBounds)
+                                   : physicalBounds / currentScaleFactor;
     }
 
     void updateBorderSize()
     {
         if ((styleFlags & windowHasTitleBar) == 0)
-            windowBorder = {};
-        else if (windowBorder.getTopAndBottom() == 0 && windowBorder.getLeftAndRight() == 0)
-            windowBorder = XWindowSystem::getInstance()->getBorderSize (windowH);
+        {
+            windowBorder = ComponentPeer::OptionalBorderSize { BorderSize<int>() };
+        }
+        else if (! windowBorder
+                 || ((*windowBorder).getTopAndBottom() == 0 && (*windowBorder).getLeftAndRight() == 0))
+        {
+            windowBorder = [&]()
+            {
+                if (auto unscaledBorderSize = XWindowSystem::getInstance()->getBorderSize (windowH))
+                    return OptionalBorderSize { (*unscaledBorderSize).multipliedBy (1.0 / currentScaleFactor) };
+
+                return OptionalBorderSize {};
+            }();
+        }
     }
 
     //==============================================================================
@@ -377,12 +446,31 @@ private:
 
             if (! totalArea.isEmpty())
             {
-                if (image.isNull() || image.getWidth() < totalArea.getWidth()
+                const auto wasImageNull = image.isNull();
+
+                if (wasImageNull || image.getWidth() < totalArea.getWidth()
                      || image.getHeight() < totalArea.getHeight())
                 {
                     image = XWindowSystem::getInstance()->createImage (isSemiTransparentWindow,
                                                                        totalArea.getWidth(), totalArea.getHeight(),
                                                                        useARGBImagesForRendering);
+                    if (wasImageNull)
+                    {
+                        // After calling createImage() XWindowSystem::getWindowBounds() will return
+                        // changed coordinates that look like the result of some position
+                        // defaulting mechanism. If we handle a configureNotifyEvent after
+                        // createImage() and before we would issue new, valid coordinates, we will
+                        // apply these default, unwanted coordinates to our window. To avoid that
+                        // we immediately send another positioning message to guarantee that the
+                        // next configureNotifyEvent will read valid values.
+                        //
+                        // This issue only occurs right after peer creation, when the image is
+                        // null. Updating when only the width or height is changed would lead to
+                        // incorrect behaviour.
+                        peer.forceSetBounds (ScalingHelpers::scaledScreenPosToUnscaled (peer.component,
+                                                                                        peer.component.getBoundsInParent()),
+                                             peer.isFullScreen());
+                    }
                 }
 
                 startTimer (repaintTimerPeriod);
@@ -425,29 +513,53 @@ private:
     };
 
     //==============================================================================
+    template <typename This>
+    static Point<float> localToGlobal (This& t, Point<float> relativePosition)
+    {
+        return relativePosition + t.getScreenPosition (false).toFloat();
+    }
+
+    template <typename This>
+    static Point<float> globalToLocal (This& t, Point<float> screenPosition)
+    {
+        return screenPosition - t.getScreenPosition (false).toFloat();
+    }
+
+    //==============================================================================
+    void settingChanged (const XWindowSystemUtilities::XSetting& settingThatHasChanged) override
+    {
+        static StringArray possibleSettings { XWindowSystem::getWindowScalingFactorSettingName(),
+                                              "Gdk/UnscaledDPI",
+                                              "Xft/DPI" };
+
+        if (possibleSettings.contains (settingThatHasChanged.name))
+            forceDisplayUpdate();
+    }
+
     void updateScaleFactorFromNewBounds (const Rectangle<int>& newBounds, bool isPhysical)
     {
-        if (! JUCEApplicationBase::isStandaloneApp())
-            return;
-
         Point<int> translation = (parentWindow != 0 ? getScreenPosition (isPhysical) : Point<int>());
+        const auto& desktop = Desktop::getInstance();
 
-        auto newScaleFactor = Desktop::getInstance().getDisplays().findDisplayForRect (newBounds.translated (translation.x, translation.y), isPhysical).scale
-                                 / Desktop::getInstance().getGlobalScaleFactor();
-
-        if (! approximatelyEqual (newScaleFactor, currentScaleFactor))
+        if (auto* display = desktop.getDisplays().getDisplayForRect (newBounds.translated (translation.x, translation.y),
+                                                                     isPhysical))
         {
-            currentScaleFactor = newScaleFactor;
-            scaleFactorListeners.call ([&] (ScaleFactorListener& l) { l.nativeScaleFactorChanged (currentScaleFactor); });
+            auto newScaleFactor = display->scale / desktop.getGlobalScaleFactor();
+
+            if (! approximatelyEqual (newScaleFactor, currentScaleFactor))
+            {
+                currentScaleFactor = newScaleFactor;
+                scaleFactorListeners.call ([&] (ScaleFactorListener& l) { l.nativeScaleFactorChanged (currentScaleFactor); });
+            }
         }
     }
 
     //==============================================================================
     std::unique_ptr<LinuxRepaintManager> repainter;
 
-    WindowHandleType windowH = {}, parentWindow = {}, keyProxy = {};
+    ::Window windowH = {}, parentWindow = {};
     Rectangle<int> bounds;
-    BorderSize<int> windowBorder;
+    ComponentPeer::OptionalBorderSize windowBorder;
     bool fullScreen = false, isAlwaysOnTop = false;
     double currentScaleFactor = 1.0;
     Array<Component*> glRepaintListeners;
@@ -456,17 +568,16 @@ private:
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (LinuxComponentPeer)
 };
 
-template<typename WindowHandleType>
-bool LinuxComponentPeer<WindowHandleType>::isActiveApplication = false;
+bool LinuxComponentPeer::isActiveApplication = false;
 
 //==============================================================================
 ComponentPeer* Component::createNewPeer (int styleFlags, void* nativeWindowToAttachTo)
 {
-    return new LinuxComponentPeer<::Window> (*this, styleFlags, (::Window) nativeWindowToAttachTo);
+    return new LinuxComponentPeer (*this, styleFlags, (::Window) nativeWindowToAttachTo);
 }
 
 //==============================================================================
-JUCE_API bool JUCE_CALLTYPE Process::isForegroundProcess()    { return LinuxComponentPeer<::Window>::isActiveApplication; }
+JUCE_API bool JUCE_CALLTYPE Process::isForegroundProcess()    { return LinuxComponentPeer::isActiveApplication; }
 
 JUCE_API void JUCE_CALLTYPE Process::makeForegroundProcess()  {}
 JUCE_API void JUCE_CALLTYPE Process::hide()                   {}
@@ -475,15 +586,18 @@ JUCE_API void JUCE_CALLTYPE Process::hide()                   {}
 void Desktop::setKioskComponent (Component* comp, bool enableOrDisable, bool)
 {
     if (enableOrDisable)
-        comp->setBounds (getDisplays().findDisplayForRect (comp->getScreenBounds()).totalArea);
+        comp->setBounds (getDisplays().getDisplayForRect (comp->getScreenBounds())->totalArea);
 }
 
 void Displays::findDisplays (float masterScale)
 {
-    displays = XWindowSystem::getInstance()->findDisplays (masterScale);
+    if (XWindowSystem::getInstance()->getDisplay() != nullptr)
+    {
+        displays = XWindowSystem::getInstance()->findDisplays (masterScale);
 
-    if (! displays.isEmpty())
-        updateToLogical();
+        if (! displays.isEmpty())
+            updateToLogical();
+    }
 }
 
 bool Desktop::canUseSemiTransparentWindows() noexcept
@@ -491,6 +605,55 @@ bool Desktop::canUseSemiTransparentWindows() noexcept
     return XWindowSystem::getInstance()->canUseSemiTransparentWindows();
 }
 
+class Desktop::NativeDarkModeChangeDetectorImpl  : private XWindowSystemUtilities::XSettings::Listener
+{
+public:
+    NativeDarkModeChangeDetectorImpl()
+    {
+        const auto* windowSystem = XWindowSystem::getInstance();
+
+        if (auto* xSettings = windowSystem->getXSettings())
+            xSettings->addListener (this);
+
+        darkModeEnabled = windowSystem->isDarkModeActive();
+    }
+
+    ~NativeDarkModeChangeDetectorImpl() override
+    {
+        if (auto* windowSystem = XWindowSystem::getInstanceWithoutCreating())
+            if (auto* xSettings = windowSystem->getXSettings())
+                xSettings->removeListener (this);
+    }
+
+    bool isDarkModeEnabled() const noexcept  { return darkModeEnabled; }
+
+private:
+    void settingChanged (const XWindowSystemUtilities::XSetting& settingThatHasChanged) override
+    {
+        if (settingThatHasChanged.name == XWindowSystem::getThemeNameSettingName())
+        {
+            const auto wasDarkModeEnabled = std::exchange (darkModeEnabled, XWindowSystem::getInstance()->isDarkModeActive());
+
+            if (darkModeEnabled != wasDarkModeEnabled)
+                Desktop::getInstance().darkModeChanged();
+        }
+    }
+
+    bool darkModeEnabled = false;
+
+    JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (NativeDarkModeChangeDetectorImpl)
+};
+
+std::unique_ptr<Desktop::NativeDarkModeChangeDetectorImpl> Desktop::createNativeDarkModeChangeDetectorImpl()
+{
+    return std::make_unique<NativeDarkModeChangeDetectorImpl>();
+}
+
+bool Desktop::isDarkModeActive() const
+{
+    return nativeDarkModeChangeDetectorImpl->isDarkModeEnabled();
+}
+
 static bool screenSaverAllowed = true;
 
 void Desktop::setScreenSaverEnabled (bool isEnabled)
@@ -540,38 +703,58 @@ void MouseInputSource::setRawMousePosition (Point<float> newPosition)
 }
 
 //==============================================================================
-void* CustomMouseCursorInfo::create() const
+class MouseCursor::PlatformSpecificHandle
 {
-    return XWindowSystem::getInstance()->createCustomMouseCursorInfo (image, hotspot);
-}
+public:
+    explicit PlatformSpecificHandle (const MouseCursor::StandardCursorType type)
+        : cursorHandle (makeHandle (type)) {}
 
-void MouseCursor::deleteMouseCursor (void* cursorHandle, bool)
-{
-    if (cursorHandle != nullptr)
-        XWindowSystem::getInstance()->deleteMouseCursor (cursorHandle);
-}
+    explicit PlatformSpecificHandle (const CustomMouseCursorInfo& info)
+        : cursorHandle (makeHandle (info)) {}
 
-void* MouseCursor::createStandardMouseCursor (MouseCursor::StandardCursorType type)
-{
-    return XWindowSystem::getInstance()->createStandardMouseCursor (type);
-}
+    ~PlatformSpecificHandle()
+    {
+        if (cursorHandle != Cursor{})
+            XWindowSystem::getInstance()->deleteMouseCursor (cursorHandle);
+    }
 
-void MouseCursor::showInWindow (ComponentPeer* peer) const
-{
-    if (peer != nullptr)
-        XWindowSystem::getInstance()->showCursor ((::Window) peer->getNativeHandle(), getHandle());
-}
+    static void showInWindow (PlatformSpecificHandle* handle, ComponentPeer* peer)
+    {
+        const auto cursor = handle != nullptr ? handle->cursorHandle : Cursor{};
+
+        if (peer != nullptr)
+            XWindowSystem::getInstance()->showCursor ((::Window) peer->getNativeHandle(), cursor);
+    }
+
+private:
+    static Cursor makeHandle (const CustomMouseCursorInfo& info)
+    {
+        const auto image = info.image.getImage();
+        return XWindowSystem::getInstance()->createCustomMouseCursorInfo (image.rescaled ((int) (image.getWidth()  / info.image.getScale()),
+                                                                                          (int) (image.getHeight() / info.image.getScale())), info.hotspot);
+    }
+
+    static Cursor makeHandle (MouseCursor::StandardCursorType type)
+    {
+        return XWindowSystem::getInstance()->createStandardMouseCursor (type);
+    }
+
+    Cursor cursorHandle;
+
+    //==============================================================================
+    JUCE_DECLARE_NON_COPYABLE (PlatformSpecificHandle)
+    JUCE_DECLARE_NON_MOVEABLE (PlatformSpecificHandle)
+};
 
 //==============================================================================
-template<typename WindowHandleType>
-static LinuxComponentPeer<WindowHandleType>* getPeerForDragEvent (Component* sourceComp)
+static LinuxComponentPeer* getPeerForDragEvent (Component* sourceComp)
 {
     if (sourceComp == nullptr)
         if (auto* draggingSource = Desktop::getInstance().getDraggingMouseSource (0))
             sourceComp = draggingSource->getComponentUnderMouse();
 
     if (sourceComp != nullptr)
-        if (auto* lp = dynamic_cast<LinuxComponentPeer<::Window>*> (sourceComp->getPeer()))
+        if (auto* lp = dynamic_cast<LinuxComponentPeer*> (sourceComp->getPeer()))
             return lp;
 
     jassertfalse;  // This method must be called in response to a component's mouseDown or mouseDrag event!
@@ -584,7 +767,7 @@ bool DragAndDropContainer::performExternalDragDropOfFiles (const StringArray& fi
     if (files.isEmpty())
         return false;
 
-    if (auto* peer = getPeerForDragEvent<::Window> (sourceComp))
+    if (auto* peer = getPeerForDragEvent (sourceComp))
         return XWindowSystem::getInstance()->externalDragFileInit (peer, files, canMoveFiles, std::move (callback));
 
     // This method must be called in response to a component's mouseDown or mouseDrag event!
@@ -598,7 +781,7 @@ bool DragAndDropContainer::performExternalDragDropOfText (const String& text, Co
     if (text.isEmpty())
         return false;
 
-    if (auto* peer = getPeerForDragEvent<::Window> (sourceComp))
+    if (auto* peer = getPeerForDragEvent (sourceComp))
         return XWindowSystem::getInstance()->externalDragTextInit (peer, text, std::move (callback));
 
     // This method must be called in response to a component's mouseDown or mouseDrag event!
@@ -629,16 +812,86 @@ void LookAndFeel::playAlertSound()
 }
 
 //==============================================================================
+static int showDialog (const MessageBoxOptions& options,
+                       ModalComponentManager::Callback* callback,
+                       Async async)
+{
+    const auto dummyCallback = [] (int) {};
+
+    switch (options.getNumButtons())
+    {
+        case 2:
+        {
+            if (async == Async::yes && callback == nullptr)
+                callback = ModalCallbackFunction::create (dummyCallback);
+
+            return AlertWindow::showOkCancelBox (options.getIconType(),
+                                                 options.getTitle(),
+                                                 options.getMessage(),
+                                                 options.getButtonText (0),
+                                                 options.getButtonText (1),
+                                                 options.getAssociatedComponent(),
+                                                 callback) ? 1 : 0;
+        }
+
+        case 3:
+        {
+            if (async == Async::yes && callback == nullptr)
+                callback = ModalCallbackFunction::create (dummyCallback);
+
+            return AlertWindow::showYesNoCancelBox (options.getIconType(),
+                                                    options.getTitle(),
+                                                    options.getMessage(),
+                                                    options.getButtonText (0),
+                                                    options.getButtonText (1),
+                                                    options.getButtonText (2),
+                                                    options.getAssociatedComponent(),
+                                                    callback);
+        }
+
+        case 1:
+        default:
+            break;
+    }
+
+   #if JUCE_MODAL_LOOPS_PERMITTED
+    if (async == Async::no)
+    {
+        AlertWindow::showMessageBox (options.getIconType(),
+                                     options.getTitle(),
+                                     options.getMessage(),
+                                     options.getButtonText (0),
+                                     options.getAssociatedComponent());
+    }
+    else
+   #endif
+    {
+        AlertWindow::showMessageBoxAsync (options.getIconType(),
+                                          options.getTitle(),
+                                          options.getMessage(),
+                                          options.getButtonText (0),
+                                          options.getAssociatedComponent(),
+                                          callback);
+    }
+
+    return 0;
+}
+
 #if JUCE_MODAL_LOOPS_PERMITTED
-void JUCE_CALLTYPE NativeMessageBox::showMessageBox (AlertWindow::AlertIconType iconType,
+void JUCE_CALLTYPE NativeMessageBox::showMessageBox (MessageBoxIconType iconType,
                                                      const String& title, const String& message,
-                                                     Component*)
+                                                     Component* /*associatedComponent*/)
 {
     AlertWindow::showMessageBox (iconType, title, message);
 }
+
+int JUCE_CALLTYPE NativeMessageBox::show (const MessageBoxOptions& options)
+{
+    return showDialog (options, nullptr, Async::no);
+}
 #endif
 
-void JUCE_CALLTYPE NativeMessageBox::showMessageBoxAsync (AlertWindow::AlertIconType iconType,
+void JUCE_CALLTYPE NativeMessageBox::showMessageBoxAsync (MessageBoxIconType iconType,
                                                           const String& title, const String& message,
                                                           Component* associatedComponent,
                                                           ModalComponentManager::Callback* callback)
@@ -646,7 +899,7 @@ void JUCE_CALLTYPE NativeMessageBox::showMessageBoxAsync (AlertWindow::AlertIcon
     AlertWindow::showMessageBoxAsync (iconType, title, message, {}, associatedComponent, callback);
 }
 
-bool JUCE_CALLTYPE NativeMessageBox::showOkCancelBox (AlertWindow::AlertIconType iconType,
+bool JUCE_CALLTYPE NativeMessageBox::showOkCancelBox (MessageBoxIconType iconType,
                                                       const String& title, const String& message,
                                                       Component* associatedComponent,
                                                       ModalComponentManager::Callback* callback)
@@ -654,7 +907,7 @@ bool JUCE_CALLTYPE NativeMessageBox::showOkCancelBox (AlertWindow::AlertIconType
     return AlertWindow::showOkCancelBox (iconType, title, message, {}, {}, associatedComponent, callback);
 }
 
-int JUCE_CALLTYPE NativeMessageBox::showYesNoCancelBox (AlertWindow::AlertIconType iconType,
+int JUCE_CALLTYPE NativeMessageBox::showYesNoCancelBox (MessageBoxIconType iconType,
                                                         const String& title, const String& message,
                                                         Component* associatedComponent,
                                                         ModalComponentManager::Callback* callback)
@@ -663,30 +916,44 @@ int JUCE_CALLTYPE NativeMessageBox::showYesNoCancelBox (AlertWindow::AlertIconTy
                                             associatedComponent, callback);
 }
 
-int JUCE_CALLTYPE NativeMessageBox::showYesNoBox (AlertWindow::AlertIconType iconType,
+int JUCE_CALLTYPE NativeMessageBox::showYesNoBox (MessageBoxIconType iconType,
                                                   const String& title, const String& message,
                                                   Component* associatedComponent,
                                                   ModalComponentManager::Callback* callback)
 {
-    return AlertWindow::showOkCancelBox (iconType, title, message, TRANS ("Yes"), TRANS ("No"),
+    return AlertWindow::showOkCancelBox (iconType, title, message, TRANS("Yes"), TRANS("No"),
                                          associatedComponent, callback);
 }
 
+void JUCE_CALLTYPE NativeMessageBox::showAsync (const MessageBoxOptions& options,
+                                                ModalComponentManager::Callback* callback)
+{
+    showDialog (options, callback, Async::yes);
+}
+
+void JUCE_CALLTYPE NativeMessageBox::showAsync (const MessageBoxOptions& options,
+                                                std::function<void (int)> callback)
+{
+    showAsync (options, ModalCallbackFunction::create (callback));
+}
+
 //==============================================================================
 Image juce_createIconForFile (const File&)
 {
     return {};
 }
 
+void juce_LinuxAddRepaintListener (ComponentPeer* peer, Component* dummy);
 void juce_LinuxAddRepaintListener (ComponentPeer* peer, Component* dummy)
 {
-    if (auto* linuxPeer = dynamic_cast<LinuxComponentPeer<::Window>*> (peer))
+    if (auto* linuxPeer = dynamic_cast<LinuxComponentPeer*> (peer))
         linuxPeer->addOpenGLRepaintListener (dummy);
 }
 
+void juce_LinuxRemoveRepaintListener (ComponentPeer* peer, Component* dummy);
 void juce_LinuxRemoveRepaintListener (ComponentPeer* peer, Component* dummy)
 {
-    if (auto* linuxPeer = dynamic_cast<LinuxComponentPeer<::Window>*> (peer))
+    if (auto* linuxPeer = dynamic_cast<LinuxComponentPeer*> (peer))
         linuxPeer->removeOpenGLRepaintListener (dummy);
 }
 
