@@ -2,15 +2,15 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
-   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-6-licence
+   End User License Agreement: www.juce.com/juce-7-licence
    Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
@@ -44,20 +44,6 @@ namespace juce
 class JUCE_API  AlertWindow  : public TopLevelWindow
 {
 public:
-    //==============================================================================
-    /** The type of icon to show in the dialog box. */
-    enum AlertIconType
-    {
-        NoIcon,         /**< No icon will be shown on the dialog box. */
-        QuestionIcon,   /**< A question-mark icon, for dialog boxes that need the
-                             user to answer a question. */
-        WarningIcon,    /**< An exclamation mark to indicate that the dialog is a
-                             warning about something and shouldn't be ignored. */
-        InfoIcon        /**< An icon that indicates that the dialog box is just
-                             giving the user some information, which doesn't require
-                             a response from them. */
-    };
-
     //==============================================================================
     /** Creates an AlertWindow.
 
@@ -71,7 +57,7 @@ public:
     */
     AlertWindow (const String& title,
                  const String& message,
-                 AlertIconType iconType,
+                 MessageBoxIconType iconType,
                  Component* associatedComponent = nullptr);
 
     /** Destroys the AlertWindow */
@@ -80,7 +66,7 @@ public:
     //==============================================================================
     /** Returns the type of alert icon that was specified when the window
         was created. */
-    AlertIconType getAlertType() const noexcept             { return alertIconType; }
+    MessageBoxIconType getAlertType() const noexcept  { return alertIconType; }
 
     //==============================================================================
     /** Changes the dialog box's message.
@@ -226,8 +212,6 @@ public:
     bool containsAnyExtraComponents() const;
 
     //==============================================================================
-    // easy-to-use message box functions:
-
    #if JUCE_MODAL_LOOPS_PERMITTED
     /** Shows a dialog box that just has a message and a single button to get rid of it.
 
@@ -244,13 +228,57 @@ public:
                             alert window should be associated with. Depending on the look
                             and feel, this might be used for positioning of the alert window.
     */
-    static void JUCE_CALLTYPE showMessageBox (AlertIconType iconType,
+    static void JUCE_CALLTYPE showMessageBox (MessageBoxIconType iconType,
                                               const String& title,
                                               const String& message,
                                               const String& buttonText = String(),
                                               Component* associatedComponent = nullptr);
+
+    /** Shows a dialog box using the specified options.
+
+        The box is shown modally, and the method will block until the user dismisses it.
+
+        @param options  the options to use when creating the dialog.
+
+        @returns  the index of the button that was clicked.
+
+        @see MessageBoxOptions
+    */
+    static int JUCE_CALLTYPE show (const MessageBoxOptions& options);
    #endif
 
+    /** Shows a dialog box using the specified options.
+
+        The box will be displayed and placed into a modal state, but this method will return
+        immediately, and the callback will be invoked later when the user dismisses the box.
+
+        @param options   the options to use when creating the dialog.
+        @param callback  if this is non-null, the callback will receive a call to its
+                         modalStateFinished() when the box is dismissed with the index of the
+                         button that was clicked as its argument.
+                         The callback object will be owned and deleted by the system, so make sure
+                         that it works safely and doesn't keep any references to objects that might
+                         be deleted before it gets called.
+
+        @see MessageBoxOptions
+    */
+    static void JUCE_CALLTYPE showAsync (const MessageBoxOptions& options,
+                                         ModalComponentManager::Callback* callback);
+
+    /** Shows a dialog box using the specified options.
+
+        The box will be displayed and placed into a modal state, but this method will return
+        immediately, and the callback will be invoked later when the user dismisses the box.
+
+        @param options   the options to use when creating the dialog.
+        @param callback  if this is non-null, the callback will be called when the box is
+                         dismissed with the index of the button that was clicked as its argument.
+
+        @see MessageBoxOptions
+    */
+    static void JUCE_CALLTYPE showAsync (const MessageBoxOptions& options,
+                                         std::function<void (int)> callback);
+
     /** Shows a dialog box that just has a message and a single button to get rid of it.
 
         The box will be displayed and placed into a modal state, but this method will
@@ -272,7 +300,7 @@ public:
                             safely and doesn't keep any references to objects that might be deleted
                             before it gets called.
     */
-    static void JUCE_CALLTYPE showMessageBoxAsync (AlertIconType iconType,
+    static void JUCE_CALLTYPE showMessageBoxAsync (MessageBoxIconType iconType,
                                                    const String& title,
                                                    const String& message,
                                                    const String& buttonText = String(),
@@ -314,20 +342,20 @@ public:
                  is not null, the method always returns false, and the user's choice is delivered
                  later by the callback.
     */
-    static bool JUCE_CALLTYPE showOkCancelBox (AlertIconType iconType,
+    static bool JUCE_CALLTYPE showOkCancelBox (MessageBoxIconType iconType,
                                                const String& title,
                                                const String& message,
-                                            #if JUCE_MODAL_LOOPS_PERMITTED
+                                              #if JUCE_MODAL_LOOPS_PERMITTED
                                                const String& button1Text = String(),
                                                const String& button2Text = String(),
                                                Component* associatedComponent = nullptr,
                                                ModalComponentManager::Callback* callback = nullptr);
-                                            #else
+                                              #else
                                                const String& button1Text,
                                                const String& button2Text,
                                                Component* associatedComponent,
                                                ModalComponentManager::Callback* callback);
-                                            #endif
+                                              #endif
 
     /** Shows a dialog box with three buttons.
 
@@ -368,24 +396,25 @@ public:
                  - 1 if the first button was pressed (normally used for 'yes')
                  - 2 if the middle button was pressed (normally used for 'no')
     */
-    static int JUCE_CALLTYPE showYesNoCancelBox (AlertIconType iconType,
+    static int JUCE_CALLTYPE showYesNoCancelBox (MessageBoxIconType iconType,
                                                  const String& title,
                                                  const String& message,
-                                               #if JUCE_MODAL_LOOPS_PERMITTED
+                                                #if JUCE_MODAL_LOOPS_PERMITTED
                                                  const String& button1Text = String(),
                                                  const String& button2Text = String(),
                                                  const String& button3Text = String(),
                                                  Component* associatedComponent = nullptr,
                                                  ModalComponentManager::Callback* callback = nullptr);
-                                               #else
+                                                #else
                                                  const String& button1Text,
                                                  const String& button2Text,
                                                  const String& button3Text,
                                                  Component* associatedComponent,
                                                  ModalComponentManager::Callback* callback);
-                                               #endif
+                                                #endif
 
     //==============================================================================
+   #if JUCE_MODAL_LOOPS_PERMITTED && ! defined (DOXYGEN)
     /** Shows an operating-system native dialog box.
 
         @param title        the title to use at the top
@@ -394,7 +423,7 @@ public:
                             it'll show a box with just an ok button
         @returns true if the ok button was pressed, false if they pressed cancel.
     */
-   #if JUCE_MODAL_LOOPS_PERMITTED
+    [[deprecated ("Use the NativeMessageBox methods instead for more options")]]
     static bool JUCE_CALLTYPE showNativeDialogBox (const String& title,
                                                    const String& bodyText,
                                                    bool isOkCancel);
@@ -428,7 +457,7 @@ public:
                                                 const String& button1,
                                                 const String& button2,
                                                 const String& button3,
-                                                AlertWindow::AlertIconType iconType,
+                                                MessageBoxIconType iconType,
                                                 int numButtons,
                                                 Component* associatedComponent) = 0;
 
@@ -444,6 +473,14 @@ public:
         virtual Font getAlertWindowFont() = 0;
     };
 
+    //==============================================================================
+    using AlertIconType = MessageBoxIconType;
+
+    static constexpr auto NoIcon       = MessageBoxIconType::NoIcon;
+    static constexpr auto QuestionIcon = MessageBoxIconType::QuestionIcon;
+    static constexpr auto WarningIcon  = MessageBoxIconType::WarningIcon;
+    static constexpr auto InfoIcon     = MessageBoxIconType::InfoIcon;
+
 protected:
     //==============================================================================
     /** @internal */
@@ -460,12 +497,15 @@ protected:
     void userTriedToCloseWindow() override;
     /** @internal */
     int getDesktopWindowStyleFlags() const override;
+    /** @internal */
+    float getDesktopScaleFactor() const override { return desktopScale * Desktop::getInstance().getGlobalScaleFactor(); }
 
 private:
     //==============================================================================
     String text;
     TextLayout textLayout;
-    AlertIconType alertIconType;
+    Label accessibleMessageLabel;
+    MessageBoxIconType alertIconType;
     ComponentBoundsConstrainer constrainer;
     ComponentDragger dragger;
     Rectangle<int> textArea;
@@ -479,7 +519,9 @@ private:
     StringArray textboxNames, comboBoxNames;
     Component* const associatedComponent;
     bool escapeKeyCancels = true;
+    float desktopScale = 1.0f;
 
+    std::unique_ptr<AccessibilityHandler> createAccessibilityHandler() override;
     void exitAlert (Button* button);
     void updateLayout (bool onlyIncreaseSize);
 
