@@ -2,15 +2,15 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
-   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-6-licence
+   End User License Agreement: www.juce.com/juce-7-licence
    Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
@@ -32,17 +32,24 @@ TooltipWindow::TooltipWindow (Component* parentComp, int delayMs)
 {
     setAlwaysOnTop (true);
     setOpaque (true);
+    setAccessible (false);
 
     if (parentComp != nullptr)
         parentComp->addChildComponent (this);
 
-    if (Desktop::getInstance().getMainMouseSource().canHover())
+    auto& desktop = Desktop::getInstance();
+
+    if (desktop.getMainMouseSource().canHover())
+    {
+        desktop.addGlobalMouseListener (this);
         startTimer (123);
+    }
 }
 
 TooltipWindow::~TooltipWindow()
 {
     hideTip();
+    Desktop::getInstance().removeGlobalMouseListener (this);
 }
 
 void TooltipWindow::setMillisecondsBeforeTipAppears (const int newTimeMs) noexcept
@@ -55,9 +62,22 @@ void TooltipWindow::paint (Graphics& g)
     getLookAndFeel().drawTooltip (g, tipShowing, getWidth(), getHeight());
 }
 
-void TooltipWindow::mouseEnter (const MouseEvent&)
+void TooltipWindow::mouseEnter (const MouseEvent& e)
 {
-    hideTip();
+    if (e.eventComponent == this)
+        hideTip();
+}
+
+void TooltipWindow::mouseDown (const MouseEvent&)
+{
+    if (isVisible())
+        dismissalMouseEventOccurred = true;
+}
+
+void TooltipWindow::mouseWheelMove (const MouseEvent&, const MouseWheelDetails&)
+{
+    if (isVisible())
+        dismissalMouseEventOccurred = true;
 }
 
 void TooltipWindow::updatePosition (const String& tip, Point<int> pos, Rectangle<int> parentArea)
@@ -74,6 +94,11 @@ void TooltipWindow::displayTip (Point<int> screenPos, const String& tip)
 {
     jassert (tip.isNotEmpty());
 
+    displayTipInternal (screenPos, tip, ShownManually::yes);
+}
+
+void TooltipWindow::displayTipInternal (Point<int> screenPos, const String& tip, ShownManually shownManually)
+{
     if (! reentrant)
     {
         ScopedValueSetter<bool> setter (reentrant, true, false);
@@ -91,12 +116,14 @@ void TooltipWindow::displayTip (Point<int> screenPos, const String& tip)
         }
         else
         {
-            updatePosition (tip, screenPos, Desktop::getInstance().getDisplays().findDisplayForPoint (screenPos).userArea);
+            const auto physicalPos = ScalingHelpers::scaledScreenPosToUnscaled (screenPos);
+            const auto scaledPos = ScalingHelpers::unscaledScreenPosToScaled (*this, physicalPos);
+            updatePosition (tip, scaledPos, Desktop::getInstance().getDisplays().getDisplayForPoint (screenPos)->userArea);
 
             addToDesktop (ComponentPeer::windowHasDropShadow
-                            | ComponentPeer::windowIsTemporary
-                            | ComponentPeer::windowIgnoresKeyPresses
-                            | ComponentPeer::windowIgnoresMouseClicks);
+                          | ComponentPeer::windowIsTemporary
+                          | ComponentPeer::windowIgnoresKeyPresses
+                          | ComponentPeer::windowIgnoresMouseClicks);
         }
 
        #if JUCE_DEBUG
@@ -106,7 +133,7 @@ void TooltipWindow::displayTip (Point<int> screenPos, const String& tip)
 
         for (auto* w : activeTooltipWindows)
         {
-            if (w != this && w->tipShowing == tipShowing && w->getParentComponent() == parent)
+            if (w != nullptr && w != this && w->tipShowing == tipShowing && w->getParentComponent() == parent)
             {
                 // Looks like you have more than one TooltipWindow showing the same tip..
                 // Be careful not to create more than one instance of this class with the
@@ -117,12 +144,14 @@ void TooltipWindow::displayTip (Point<int> screenPos, const String& tip)
        #endif
 
         toFront (false);
+        manuallyShownTip = shownManually == ShownManually::yes ? tip : String();
+        dismissalMouseEventOccurred = false;
     }
 }
 
 String TooltipWindow::getTipFor (Component& c)
 {
-    if (Process::isForegroundProcess()
+    if (isForegroundOrEmbeddedProcess (&c)
          && ! ModifierKeys::currentModifiers.isAnyMouseButtonDown())
     {
         if (auto* ttc = dynamic_cast<TooltipClient*> (&c))
@@ -135,72 +164,89 @@ String TooltipWindow::getTipFor (Component& c)
 
 void TooltipWindow::hideTip()
 {
-    if (! reentrant)
+    if (isVisible() && ! reentrant)
     {
-        tipShowing.clear();
+        tipShowing = {};
+        manuallyShownTip = {};
+        dismissalMouseEventOccurred = false;
+
         removeFromDesktop();
         setVisible (false);
 
+        lastHideTime = Time::getApproximateMillisecondCounter();
+
        #if JUCE_DEBUG
         activeTooltipWindows.removeAllInstancesOf (this);
        #endif
     }
 }
 
+float TooltipWindow::getDesktopScaleFactor() const
+{
+    if (lastComponentUnderMouse != nullptr)
+        return Component::getApproximateScaleFactorForComponent (lastComponentUnderMouse);
+
+    return Component::getDesktopScaleFactor();
+}
+
+std::unique_ptr<AccessibilityHandler> TooltipWindow::createAccessibilityHandler()
+{
+    return createIgnoredAccessibilityHandler (*this);
+}
+
 void TooltipWindow::timerCallback()
 {
-    auto& desktop = Desktop::getInstance();
-    auto mouseSource = desktop.getMainMouseSource();
-    auto now = Time::getApproximateMillisecondCounter();
-
+    const auto mouseSource = Desktop::getInstance().getMainMouseSource();
     auto* newComp = mouseSource.isTouch() ? nullptr : mouseSource.getComponentUnderMouse();
 
+    if (manuallyShownTip.isNotEmpty())
+    {
+        if (dismissalMouseEventOccurred || newComp == nullptr)
+            hideTip();
+
+        return;
+    }
+
     if (newComp == nullptr || getParentComponent() == nullptr || newComp->getPeer() == getPeer())
     {
-        auto newTip = newComp != nullptr ? getTipFor (*newComp) : String();
-        bool tipChanged = (newTip != lastTipUnderMouse || newComp != lastComponentUnderMouse);
+        const auto newTip = newComp != nullptr ? getTipFor (*newComp) : String();
+
+        const auto mousePos = mouseSource.getScreenPosition();
+        const auto mouseMovedQuickly = (mousePos.getDistanceFrom (lastMousePos) > 12);
+        lastMousePos = mousePos;
+
+        const auto tipChanged = (newTip != lastTipUnderMouse || newComp != lastComponentUnderMouse);
+        const auto now = Time::getApproximateMillisecondCounter();
+
         lastComponentUnderMouse = newComp;
         lastTipUnderMouse = newTip;
 
-        auto clickCount = desktop.getMouseButtonClickCounter();
-        auto wheelCount = desktop.getMouseWheelMoveCounter();
-        bool mouseWasClicked = (clickCount > mouseClicks || wheelCount > mouseWheelMoves);
-        mouseClicks = clickCount;
-        mouseWheelMoves = wheelCount;
-
-        auto mousePos = mouseSource.getScreenPosition();
-        bool mouseMovedQuickly = mousePos.getDistanceFrom (lastMousePos) > 12;
-        lastMousePos = mousePos;
-
-        if (tipChanged || mouseWasClicked || mouseMovedQuickly)
+        if (tipChanged || dismissalMouseEventOccurred || mouseMovedQuickly)
             lastCompChangeTime = now;
 
+        const auto showTip = [this, &mouseSource, &mousePos, &newTip]
+        {
+            if (mouseSource.getLastMouseDownPosition() != lastMousePos)
+                displayTipInternal (mousePos.roundToInt(), newTip, ShownManually::no);
+        };
+
         if (isVisible() || now < lastHideTime + 500)
         {
             // if a tip is currently visible (or has just disappeared), update to a new one
             // immediately if needed..
-            if (newComp == nullptr || mouseWasClicked || newTip.isEmpty())
-            {
-                if (isVisible())
-                {
-                    lastHideTime = now;
-                    hideTip();
-                }
-            }
+            if (newComp == nullptr || dismissalMouseEventOccurred || newTip.isEmpty())
+                hideTip();
             else if (tipChanged)
-            {
-                displayTip (mousePos.roundToInt(), newTip);
-            }
+                showTip();
         }
         else
         {
-            // if there isn't currently a tip, but one is needed, only let it
-            // appear after a timeout..
+            // if there isn't currently a tip, but one is needed, only let it appear after a timeout
             if (newTip.isNotEmpty()
-                 && newTip != tipShowing
-                 && now > lastCompChangeTime + (uint32) millisecondsBeforeTipAppears)
+                && newTip != tipShowing
+                && now > lastCompChangeTime + (uint32) millisecondsBeforeTipAppears)
             {
-                displayTip (mousePos.roundToInt(), newTip);
+                showTip();
             }
         }
     }
