@@ -2,15 +2,15 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
-   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-6-licence
+   End User License Agreement: www.juce.com/juce-7-licence
    Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
@@ -47,7 +47,19 @@ namespace
         io_iterator_t iter = 0;
         io_object_t iod = 0;
 
-        if (IOServiceGetMatchingServices (kIOMasterPortDefault, dict, &iter) == kIOReturnSuccess
+        const auto defaultPort = []
+        {
+           #if defined (MAC_OS_VERSION_12_0) && MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_VERSION_12_0
+            if (@available (macOS 12.0, *))
+                return kIOMainPortDefault;
+           #endif
+
+            JUCE_BEGIN_IGNORE_WARNINGS_GCC_LIKE ("-Wdeprecated-declarations")
+            return kIOMasterPortDefault;
+            JUCE_END_IGNORE_WARNINGS_GCC_LIKE
+        }();
+
+        if (IOServiceGetMatchingServices (defaultPort, dict, &iter) == kIOReturnSuccess
              && iter != 0)
         {
             iod = IOIteratorNext (iter);
@@ -143,15 +155,15 @@ bool AppleRemoteDevice::open (const bool openInExclusiveMode)
 {
     Array<int> cookies;
 
-    CFArrayRef elements;
+    CFObjectHolder<CFArrayRef> elements;
     auto device122 = (IOHIDDeviceInterface122**) device;
 
-    if ((*device122)->copyMatchingElements (device122, nullptr, &elements) != kIOReturnSuccess)
+    if ((*device122)->copyMatchingElements (device122, nullptr, &elements.object) != kIOReturnSuccess)
         return false;
 
-    for (int i = 0; i < CFArrayGetCount (elements); ++i)
+    for (int i = 0; i < CFArrayGetCount (elements.object); ++i)
     {
-        auto element = (CFDictionaryRef) CFArrayGetValueAtIndex (elements, i);
+        auto element = (CFDictionaryRef) CFArrayGetValueAtIndex (elements.object, i);
 
         // get the cookie
         CFTypeRef object = CFDictionaryGetValue (element, CFSTR (kIOHIDElementCookieKey));
@@ -166,8 +178,6 @@ bool AppleRemoteDevice::open (const bool openInExclusiveMode)
         cookies.add ((int) number);
     }
 
-    CFRelease (elements);
-
     if ((*(IOHIDDeviceInterface**) device)
             ->open ((IOHIDDeviceInterface**) device,
                     openInExclusiveMode ? kIOHIDOptionsTypeSeizeDevice
