@@ -3,7 +3,7 @@
 
    This file is part of the Water library.
    Copyright (c) 2016 ROLI Ltd.
-   Copyright (C) 2017 Filipe Coelho <falktx@falktx.com>
+   Copyright (C) 2017-2022 Filipe Coelho <falktx@falktx.com>
 
    Permission is granted to use this software under the terms of the ISC license
    http://www.isc.org/downloads/software-support-policy/isc-license/
@@ -73,25 +73,6 @@ public:
         deleteAllObjects();
     }
 
-   #if WATER_COMPILER_SUPPORTS_MOVE_SEMANTICS
-    OwnedArray (OwnedArray&& other) noexcept
-        : data (static_cast<ArrayAllocationBase <ObjectClass*>&&> (other.data)),
-          numUsed (other.numUsed)
-    {
-        other.numUsed = 0;
-    }
-
-    OwnedArray& operator= (OwnedArray&& other) noexcept
-    {
-        deleteAllObjects();
-
-        data = static_cast<ArrayAllocationBase <ObjectClass*>&&> (other.data);
-        numUsed = other.numUsed;
-        other.numUsed = 0;
-        return *this;
-    }
-   #endif
-
     //==============================================================================
     /** Clears the array, optionally deleting the objects inside it first. */
     void clear (bool deleteObjects = true)
@@ -826,7 +807,7 @@ private:
             delete data.elements [--numUsed];
     }
 
-    CARLA_DECLARE_NON_COPY_CLASS (OwnedArray)
+    CARLA_DECLARE_NON_COPYABLE (OwnedArray)
 };
 
 }
