@@ -3,7 +3,7 @@
 
    This file is part of the Water library.
    Copyright (c) 2016 ROLI Ltd.
-   Copyright (C) 2017 Filipe Coelho <falktx@falktx.com>
+   Copyright (C) 2017-2022 Filipe Coelho <falktx@falktx.com>
 
    Permission is granted to use this software under the terms of the ISC license
    http://www.isc.org/downloads/software-support-policy/isc-license/
@@ -128,7 +128,7 @@ private:
     void trimExternalBlockSize();
     char* prepareToWrite (size_t);
 
-    CARLA_DECLARE_NON_COPY_CLASS (MemoryOutputStream)
+    CARLA_DECLARE_NON_COPYABLE (MemoryOutputStream)
 };
 
 /** Copies all the data that has been written to a MemoryOutputStream into another stream. */
