@@ -1,6 +1,6 @@
 /*
  * Carla Native Plugins
- * Copyright (C) 2013-2021 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2013-2022 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,7 +22,7 @@
 
 static const char* const audiofilesWildcard =
 #ifdef HAVE_SNDFILE
-    "*.aif;*.aifc;*.aiff;*.au;*.bwf;*.flac;*.htk;*.iff;*.mat4;*.mat5;*.oga;*.ogg;"
+    "*.aif;*.aifc;*.aiff;*.au;*.bwf;*.flac;*.htk;*.iff;*.mat4;*.mat5;*.oga;*.ogg;*.opus;"
     "*.paf;*.pvf;*.pvf5;*.sd2;*.sf;*.snd;*.svx;*.vcc;*.w64;*.wav;*.xi;"
 #endif
 #ifdef HAVE_FFMPEG
@@ -751,7 +751,7 @@ private:
             }
         }
 
-        CARLA_DECLARE_NON_COPY_STRUCT(InlineDisplay)
+        CARLA_DECLARE_NON_COPYABLE(InlineDisplay)
         CARLA_PREVENT_HEAP_ALLOCATION
     } fInlineDisplay;
 #endif
@@ -835,10 +835,10 @@ static const NativePluginDescriptor audiofileDesc = {
 
 // -----------------------------------------------------------------------
 
-CARLA_EXPORT
+CARLA_API_EXPORT
 void carla_register_native_plugin_audiofile();
 
-CARLA_EXPORT
+CARLA_API_EXPORT
 void carla_register_native_plugin_audiofile()
 {
     carla_register_native_plugin(&audiofileDesc);
