@@ -26,7 +26,7 @@
 // -----------------------------------------------------------------------
 // CarlaString class
 
-class CarlaString
+class CARLA_API CarlaString
 {
 public:
     // -------------------------------------------------------------------
